from sanic.response import text
from sanic import Sanic
from sanic import response as res
from sanic.response import json

app = Sanic(__name__)

"""
multi_nlu_servant.py: 兼容rasa-nlu-1.x的http接口的servant
"""

default_return = {
        "intent": {"name": "", "confidence": 0.0},
        "entities": [],
        "text": text,
    }

@app.post('/simulate/<lang_id>/model/parse')
async def post_handler(request, lang_id):
    reqdata=request.json
    print('POST request {} - {}'.format(lang_id, reqdata))
    text=reqdata['text']

    return json(default_return)

@app.post('/nlu/<lang_id>/model/parse')
async def post_handler(request, lang_id):
    from saai.nlu_mod_procs import nlu_mods

    reqdata=request.json
    print('POST request {} - {}'.format(lang_id, reqdata))
    text=reqdata['text']

    resp = await nlu_mods.parse_async(text, lang_id)
    ret=resp if resp is not None and len(resp)>0 else default_return
    return json(ret)


if __name__ == '__main__':
    app.run(host="0.0.0.0", port=5009)




import requests

from honcho.manager import Manager
import sys
import logging
logger = logging.getLogger(__name__)

def invoke_nlu(url, text):
    headers = {
        'content-type': 'application/json',
    }
    params = {
        "text": text
    }
    try:
        result = requests.post(url, headers=headers, json=params)
        if result.status_code == 200:
            return result.json()
        else:
            logger.error(
                "Failed to parse text '{}' using rasa NLU over http. "
                "Error: {}".format(text, result.text))
            return None
    except Exception as e:
        logger.error(
            "Failed to parse text '{}' using rasa NLU over http. "
            "Error: {}".format(text, e))
        return None


class Runner(object):
    def __init__(self, start_port=2000):
        self.nlu_servants={}
        for i, lang in enumerate(['en','zh','ja','es','fr','de','ru']):
            self.nlu_servants[lang]=start_port+i

    def tests(self):
        """
        $ python -m saai.runner tests
        :return:
        """
        m = Manager()
        m.add_process('nlu_en', './startup.run nlu en 2001')
        m.add_process('nlu_zh', './startup.run nlu zh 2002')
        m.loop()

        sys.exit(m.returncode)

    def exec_spec(self, langs):
        """
        $ python -m saai.runner exec_spec zh,en,ja
        $ python -m saai.runner exec_spec zh,en,ja,de
        :param langs:
        :return:
        """
        m = Manager()
        for lang in langs:
            port=self.nlu_servants[lang]
            m.add_process(f'nlu_{lang}', f'./startup.run nlu {lang} {port}')
        m.loop()

        sys.exit(m.returncode)

    def invoke(self, sents, lang, port=0):
        """
        $ python -m saai.runner invoke "几台电脑" zh
        $ python -m saai.runner invoke "卵を食べる" ja
        $ python -m saai.runner invoke "can i get french food" en
        $ python -m saai.runner invoke "Shenzhen ist das Silicon Valley für Hardware-Firmen" de

        :param sents:
        :param lang:
        :return:
        """
        from pprint import pprint

        if port ==0:
            port = self.nlu_servants[lang]
        url=f"http://localhost:{port}/model/parse"
        r=invoke_nlu(url, sents)
        if r is not None:
            pprint(r)
            print('done.')

if __name__ == '__main__':
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(Runner)




import streamlit as st

from interacts.common import display_lang_selector
from interacts.sl_utils import all_labels, write_styles
from interacts.tracker_streamlit import enable_streamlit_tracker
import requests
from sl_utils import HTML_WRAPPER

# from sl_ent_vis import ent_vis

enable_streamlit_tracker()
write_styles()

def sidebar(sender):
    from saai.saai_bot_cli import SaaiBotCli

    cur_lang=display_lang_selector()
    opt=st.sidebar.selectbox('Functions', ['nlu parse', 'bot talk'])
    if st.sidebar.button("reset conversation"):
        SaaiBotCli().reset_conversation(sender=sender)
    return cur_lang, opt

DEFAULT_TEXT = '感觉发烧了，该去哪个诊所哪个科室呢'

def parse(sents):
    response = requests.post(f'http://localhost:5005/model/parse', json={'text': sents})
    print('status code:', response.status_code)
    return response.json()

def talk(sents, sender='default'):
    response = requests.post(f'http://localhost:5005/webhooks/rest/webhook', json={'message': sents, 'sender':sender})
    print('status code:', response.status_code)
    return response.json()

def nlu_vis(sents):
    from ipymarkup import box_markup
    result=parse(sents)
    ents = result['entities']
    spans = [(w['start'], w['end'], w['entity']) for w in ents]
    markup = box_markup(sents, spans)
    html = ''
    for line in markup.as_html:
        html = html + line
    # Newlines seem to mess with the rendering
    # html = html.replace("\n", " ")
    st.write(HTML_WRAPPER.format(html), unsafe_allow_html=True)
    st.write(result['intent'])

def bot_talk(sents, sender='default'):
    result=talk(sents, sender)
    st.markdown(f"**你说**  `{sents}`")
    st.write(result)

def main():
    sender='default'
    st.header("bot interact")
    lang, opt=sidebar(sender)

    if opt=='nlu parse':
        st.subheader("nlu parse")
        text = st.text_area("Text to analyze", DEFAULT_TEXT)
        nlu_vis(text)
    elif opt=='bot talk':
        st.subheader("nlu parse")
        text = st.text_input("Text to send", '你好')
        bot_talk(sents=text, sender=sender)

if __name__ == '__main__':
    main()




from typing import Text, Optional
from rasa.cli.utils import create_output_path, print_success
import os
import rasa
from rasa.constants import (
    DEFAULT_CONFIG_PATH,
    DEFAULT_DATA_PATH,
    DEFAULT_DOMAIN_PATH,
    DOCS_BASE_URL,
)

def scaffold_path(proto) -> Text:
    import pkg_resources

    return pkg_resources.resource_filename(__name__, f"initial_{proto}")

def create_initial_project(path: Text, proto:Text) -> None:
    from distutils.dir_util import copy_tree

    copy_tree(scaffold_path(proto), path)
    print("Created project directory at '{}'.".format(os.path.abspath(path)))

def train_project(path: Text) -> Optional[Text]:
    print_success("Training an initial model...")
    config = os.path.join(path, DEFAULT_CONFIG_PATH)
    training_files = os.path.join(path, DEFAULT_DATA_PATH)
    domain = os.path.join(path, DEFAULT_DOMAIN_PATH)
    output = os.path.join(path, create_output_path())

    model = rasa.train(domain, config, training_files, output)
    return model

def rasa_shell(model: Text):
    from rasa.core import constants
    from rasa.cli.shell import shell
    import argparse
    args=argparse.Namespace(model=model)
    # provide defaults for command line arguments
    attributes = [
        "endpoints",
        "credentials",
        "cors",
        "auth_token",
        "jwt_secret",
        "jwt_method",
        "enable_api",
        "remote_storage",
    ]
    for a in attributes:
        setattr(args, a, None)

    args.port = constants.DEFAULT_SERVER_PORT

    shell(args)




from rasa.core.trackers import DialogueStateTracker
from rasa.core.interpreter import RasaNLUHttpInterpreter
from rasa.utils.endpoints import EndpointConfig
from rasa.core.slots import Slot
from pprint import pprint

DEFAULT_SERVER_PORT = 5008
DEFAULT_SERVER_FORMAT = "{}://localhost:{}"
DEFAULT_SERVER_URL = DEFAULT_SERVER_FORMAT.format("http", DEFAULT_SERVER_PORT)

async def nlu_parse(url, message):
    tracker = DialogueStateTracker.from_dict("1", [], [Slot("requested_language")])
    # we'll expect this value 'en' to be part of the result from the interpreter
    tracker._set_slot("requested_language", "en")
    inte=RasaNLUHttpInterpreter(EndpointConfig(url))
    result = await inte.parse(message, tracker=tracker)
    return result

class MultiNluClient(object):
    async def raw_invoke(self):
        result=await nlu_parse(DEFAULT_SERVER_URL, 'hi')
        pprint(result)

    async def simulate(self):
        result = await nlu_parse('http://localhost:5009/simulate/en', 'hi')
        pprint(result)

    async def multilang(self, lang, text):
        result = await nlu_parse(f'http://localhost:5009/nlu/{lang}', text)
        return result

multi_nlu=MultiNluClient()





import json
import json_utils
import sagas.tracker_fn as tc

prefix = '/pi/ws/sagas-ai/nlu_multilang'
def train_mod(lang):
    from rasa.train import train_nlu, train_async
    from saai.tools.corpus_procs import CorpusProcs
    if lang in ('ja','zh'):
        # saai.tools.corpus_procs gen_datasets cn '/pi/ws/sagas-ai/nlu_multilang/zh/'
        CorpusProcs().gen_datasets('cn' if lang=='zh' else lang, f'{prefix}/{lang}/')
    train_nlu(
        config=f"{prefix}/config_{lang}.yml",
        nlu_data=f"{prefix}/{lang}/",
        output=f'{prefix}/models',
        fixed_model_name=f'{lang}_current',
        persist_nlu_training_data=True
    )

def load_mod(lang):
    from rasa.core.agent import Agent, load_agent
    # from rasa.nlu.model import Interpreter
    path = f'{prefix}/models/{lang}_current.tar.gz'
    # interpreter = Interpreter.load(path)
    agent = Agent.load(path)
    return agent

async def intepret(agent, text):
    interpreter = agent.interpreter
    return await interpreter.parse(text)

mods = ('de', 'en', 'es', 'fr', 'ru', 'corpus')
langs = {'de', 'en', 'fr', 'ru', 'es', 'zh', 'ja'}
class NluMods(object):
    def __init__(self):
        self.mods = {}

    async def parse_async(self, sents, lang):
        if lang not in langs:
            return {}
        if lang not in self.mods:
            self.mods[lang]=load_mod(lang)
        return await intepret(self.mods[lang], sents)

    def reload(self, lang):
        # train_mod(lang)
        self.mods[lang] = load_mod(lang)
        # return await intepret(self.mods[lang], 'hi')
        return True

class NluModProcs(object):
    def __init__(self):
        self.timestamps_file=f'{prefix}/last_timestamps.json'

    def get_timestamps(self):
        import io_utils
        import os

        last_timestamps = {}
        for mod in mods:
            folder = f"{prefix}/{mod}/"
            # print(folder)
            # print(os.listdir(folder))
            files = [(f, os.path.getmtime(f)) for f in io_utils.list_files(folder)
                     if os.path.isfile(f)]
            # print(len(files))
            files = sorted(files, key=lambda x: x[1], reverse=True)
            last_modified_file = None if len(files) == 0 else files[0][0]
            if last_modified_file is not None:
                mark = os.path.getmtime(last_modified_file)
                # print(last_modified_file, mark)
                last_timestamps[mod] = mark
        return last_timestamps

    def write_timestamps(self):
        """
        $ python -m saai.nlu_mod_procs write_timestamps
        :return:
        """
        last_timestamps = self.get_timestamps()
        print(last_timestamps)
        json_utils.write_json_to_file(self.timestamps_file, last_timestamps)

    def check_modified(self):
        """
        $ python -m saai.nlu_mod_procs check_modified
        :return:
        """
        import os

        if not os.path.exists(self.timestamps_file):
            return mods

        before = json_utils.read_json_file(self.timestamps_file)
        current = self.get_timestamps()
        modified = []
        for item in current:
            if current[item] - before[item] > 1:
                modified.append(item)
        return modified

    def train_all(self, force=False):
        """
        $ python -m saai.nlu_mod_procs train_all
        :return:
        """
        if force:
            train_set=mods
        else:
            train_set=self.check_modified()
        if len(train_set)>0:
            for lang in train_set:
                if lang=='corpus':
                    tc.emp('green', 'train zh,ja ...')
                    train_mod('zh')
                    train_mod('ja')
                else:
                    tc.emp('green', f'train {lang} ...')
                    train_mod(lang)
            self.write_timestamps()
        else:
            tc.emp('yellow', 'all up-to-date.')

    def train(self, lang):
        """
        $ python -m saai.nlu_mod_procs train zh
        $ python -m saai.nlu_mod_procs train en
        :param lang:
        :return:
        """
        train_mod(lang)

    def parse(self, sents, lang):
        """
        $ python -m saai.nlu_mod_procs parse 'hi' en

        :param sents:
        :param lang:
        :return:
        """
        import asyncio

        agent=load_mod(lang)
        loop = asyncio.get_event_loop()
        return loop.run_until_complete(intepret(agent, sents))

    def parse_pp(self, sents, lang):
        """
        $ python -m saai.nlu_mod_procs parse_pp 'hi' en
        $ python -m saai.nlu_mod_procs parse_pp '附近有什么好吃的' zh
        """
        from pprint import pprint
        pprint(self.parse(sents, lang))

nlu_mods=NluMods()

if __name__ == '__main__':
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(NluModProcs)




from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.events import UserUtteranceReverted
from rasa_sdk.executor import CollectingDispatcher
from pprint import pprint
import logging
import json

class ActionHelloWorld(Action):

    def name(self) -> Text:
        return "action_hello_world"

    def run(self, dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:

        dispatcher.utter_message("Hello World!")

        return []

class ActionIsBot(Action):
    """Revertible mapped action for utter_is_bot"""

    def name(self):
        return "action_is_bot"

    def run(self, dispatcher: CollectingDispatcher,
            tracker:Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        dump_slots(tracker)
        dispatcher.utter_template("utter_iamabot", tracker)
        return [UserUtteranceReverted()]

def prop(tracker:Tracker, attr:Text):
    attr_val = tracker.get_slot(attr) if attr in tracker.slots else ''
    return attr_val

def dump_slots(tracker):
    logging.info('.. slots')
    logging.info(json.dumps(tracker.current_slot_values(), indent=2, ensure_ascii=False))

class ActionLogCommEvent(Action):
    def name(self):
        return "action_log_commevent"

    def run(self, dispatcher: CollectingDispatcher,
            tracker:Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        dump_slots(tracker)
        dispatcher.utter_message(json_message={'result': 'log ok',
                                               'sents':prop(tracker, 'sents')})
        return [UserUtteranceReverted()]

class ActionPerformMedia(Action):
    def name(self):
        return "action_perform_media"

    def run(self, dispatcher: CollectingDispatcher,
            tracker:Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:

        dump_slots(tracker)
        dispatcher.utter_message(json_message={'result': 'success',
                                               'media_list': ['first song', 'second song'],
                                               'media_type': tracker.get_slot('object_type'),
                                               'sents':prop(tracker, 'sents')})
        return [UserUtteranceReverted()]




from durable.lang import *
from durable.engine import Host

with ruleset('verbs'):
    @when_all(m.lemma.matches('want.*'))
    def verb_want(c):
        # c.s.matches = True
        print('1. verb matches pat -> {0}'.format(c.m.text))


    @when_all(m.xpos.matches('VBP.*'))
    def verb_xpos(c):
        c.s.pos_matches = True
        print('2. verb xpos matches pat -> {0}'.format(c.m.xpos))


    @when_all(m.domains.amod.specs.anyItem(item.matches('.*thing.*, entity.*')))
    def amod_with(c):
        c.s.amod_with = True
        print('3. amod pat -> {0}'.format(c.domains.amod))


    @when_all(m.xcomp.anyItem(item.text.matches('play')))
    def xcomp_play(c):
        # print ('xcomp pat -> {0}'.format(c.m.domains.xcomp.text))
        print('4. xcomp pat')


    @when_all(m.nsubj.anyItem(item.text.matches('I')))
    def subj_play(c):
        # print ('xcomp pat -> {0}'.format(c.m.domains.xcomp.text))
        print('5. nsubj pat')

with ruleset('risk'):
    @when_all(c.first << m.t == 'purchase',
              c.second << m.location != c.first.location)
    # the event pair will only be observed once
    def fraud(c):
        print('risk-> Fraud detected -> {0}, {1}'.format(c.first.location, c.second.location))


class RulesTests(object):
    def tests_1(self):
        """
        $ python -m saai.tests.rules_tests tests_1
        :return:
        """
        v = {'nsubj': [{'index': '1', 'text': 'I'}],
             'xcomp': [{'index': '4',
                       'text': 'play',
                       'lemma': 'play',
                       'upos': 'VERB',
                       'xpos': 'VB',
                       'feats': 'VerbForm=Inf',
                       'governor': 2,
                       'dependency_relation': 'xcomp',
                       'mark': [{'index': '3',
                                'text': 'to',
                                'lemma': 'to',
                                'upos': 'PART',
                                'xpos': 'TO',
                                'feats': '_',
                                'governor': 4,
                                'dependency_relation': 'mark'}],
                       'obj': [{'index': '5',
                               'text': 'music',
                               'lemma': 'music',
                               'upos': 'NOUN',
                               'xpos': 'NN',
                               'feats': 'Number=Sing',
                               'governor': 4,
                               'dependency_relation': 'obj'}]}],
             "index": "2",
             "text": "want", "lemma": "want",
             "upos": "VERB",
             "xpos": "VBP",
             "feats": "Mood=Ind|Tense=Pres|VerbForm=Fin",
             "governor": 0, "dependency_relation": "root"}
        print(assert_fact('verbs', v))

    def tests_2(self):
        """
        $ python -m saai.tests.rules_tests tests_2
        :return:
        """
        # 'post' submits events, try 'assert' instead and to see differt behavior
        # post('risk', {'t': 'purchase', 'location': 'US'})
        # post('risk', {'t': 'purchase', 'location': 'CA'})
        post_batch('risk', [{'t': 'purchase', 'location': 'US'},{'t': 'purchase', 'location': 'CA'}])
        print('..')
        assert_fact('risk', {'t': 'purchase', 'location': 'US'})
        assert_fact('risk', {'t': 'purchase', 'location': 'CA'})

if __name__ == '__main__':
    import fire
    fire.Fire(RulesTests)




from typing import Text

import requests
from pprint import pprint

def ask_overwrite(path: Text) -> None:
    import questionary
    import os

    overwrite = questionary.confirm(
        "Directory '{}' is not empty. Continue?".format(os.path.abspath(path))
    ).ask()
    return overwrite

class SaaiBotCli(object):
    def version(self):
        """
        Show saai version info
        :return:
        """
        from saai.version import __version__
        print(f"saai version: {__version__}")

    def gen_corpus(self):
        """
        Generate corpus data
        :return:
        """
        from saai.tools.corpus_procs import CorpusProcs
        proc=CorpusProcs()
        proc.gen_local()

    def train(self):
        """
        Generate and train nlu data
        :return:
        """
        import subprocess

        self.gen_corpus()
        subprocess.run(["rasa", "train"])

    def parse(self, sents):
        """
        $ saai parse '感觉发烧了，该去哪个诊所哪个科室呢'

        :param sents:
        :return:
        """
        response = requests.post(f'http://localhost:5005/model/parse', json={'text': sents})
        print('status code:', response.status_code)
        pprint(response.json())

    def talk(self, sents, sender='default'):
        """
        $ saai talk 'hello'

        :param sender:
        :return:
        """
        response = requests.post(f'http://localhost:5005/webhooks/rest/webhook',
                                 json={'message': sents, 'sender':sender})
        print('status code:', response.status_code)
        pprint(response.json())

    def reset_conversation(self, sender='default'):
        """
        $ saai reset_conversation

        :param sender:
        :return:
        """
        response = requests.post(f'http://localhost:5005/conversations/{sender}/tracker/events',
                                 json={'event': 'restart'})
        print('status code:', response.status_code)

    def scaffold_info(self):
        """
        $ saai scaffold_info
        :return:
        """
        from saai.scaffold.project_creator import scaffold_path
        print(f"{scaffold_path()}")

    def init_proj(self, proto='en', train_it=False):
        """
        $ saai init_proj zh True
        :param proto:
        :param train_it:
        :return:
        """
        from saai.scaffold.project_creator import create_initial_project, train_project

        import os
        path=os.path.abspath('.')
        if len(os.listdir(path)) > 0:
            if not ask_overwrite(path):
                return

        create_initial_project(path, proto=proto)
        if train_it:
            if len(os.listdir("./corpus")):
                self.gen_corpus()
            train_project(path)

    def build(self, file:Text):
        """
        $ saai build servant-stack.dockerfile
        :param file:
        :return:
        """
        import subprocess
        if file.endswith('.dockerfile'):
            image_name=file.replace('.dockerfile', '').replace('-', '_')
            subprocess.run(["docker", "build",
                            '-t', f"samlet/{image_name}",
                            '-f', file, '.'])
        else:
            print(".. don't support this file type.")



# miscellaneous
AUTOMATICALLY_EXTENSIBLE = "automatically_extensible"
USE_SYNONYMS = "use_synonyms"
SYNONYMS = "synonyms"
DATA = "data"
INTENTS = "intents"
ENTITIES = "entities"
ENTITY = "entity"
ENTITY_KIND = "entity_kind"
RESOLVED_VALUE = "resolved_value"
SLOT_NAME = "slot_name"
TEXT = "text"
UTTERANCES = "utterances"
LANGUAGE = "language"
VALUE = "value"
NGRAM = "ngram"
TOKEN_INDEXES = "token_indexes"
CAPITALIZE = "capitalize"
UNKNOWNWORD = "unknownword"
VALIDATED = "validated"
START = "start"
END = "end"
BUILTIN_ENTITY_PARSER = "builtin_entity_parser"
CUSTOM_ENTITY_PARSER = "custom_entity_parser"
MATCHING_STRICTNESS = "matching_strictness"
LICENSE_INFO = "license_info"
RANDOM_STATE = "random_state"
BYPASS_VERSION_CHECK = "bypass_version_check"




import sagas

class ProcsTests(object):
    def test_df(self):
        """
        $ python -m procs_tests test_df
        :return:
        """
        sagas.print_rs([('tom', 5)], ['name', 'age'])

if __name__ == '__main__':
    import fire
    fire.Fire(ProcsTests)



from flask import Flask
from flask import request
import json

app = Flask(__name__)

@app.route("/")
def home():
    return "servant: agents"

@app.route('/message', methods = ['POST'])
def handle_message():
    """
    $ curl -XPOST -H 'Content-Type: application/json' -d '{"lang":"zh", "sents":"我是一个好老师"}'  http://localhost:15001/digest
    :return:
    """
    print("request is json?", request.is_json)
    content = request.get_json()
    mod=content['mod']
    sents=content['sents']
    lang=content['lang']
    print(f".. {mod}, {sents}, {lang}")

    data = {'lang': lang}

    # data_y=yaml.dump(data, default_flow_style=True,Dumper=KludgeDumper,encoding=None)
    data_y = json.dumps(data, ensure_ascii=False)
    return data_y

class AgentsServant(object):
    def __init__(self, port=15077):
        self.port=port

    def dev(self):
        """
        $ python -m saai.servant_agents dev
        :return:
        """
        app.run(host='0.0.0.0', port=self.port, debug=True)

    def run(self):
        from waitress import serve
        serve(app, host='0.0.0.0', port=self.port)

if __name__ == "__main__":
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(AgentsServant)




from typing import Any, Dict, List, Text
from rasa.nlu.tokenizers.tokenizer import Token, Tokenizer
from rasa.nlu.constants import (
    # MESSAGE_RESPONSE_ATTRIBUTE,
    # MESSAGE_INTENT_ATTRIBUTE,
    # MESSAGE_TEXT_ATTRIBUTE,
    # MESSAGE_TOKENS_NAMES,
    TEXT_ATTRIBUTE,
    MESSAGE_ATTRIBUTES,
    # MESSAGE_SPACY_FEATURES_NAMES,
    # MESSAGE_VECTOR_FEATURE_NAMES,
)

from sagas.util.rest_common import query_data_by_url
from rasa.nlu.training_data import TrainingData, Message
from rasa.nlu.tokenizers.whitespace_tokenizer import WhitespaceTokenizer
from sagas.conf.conf import cf

class MultilangTokenizer(WhitespaceTokenizer):
    def __init__(self, component_config: Dict[Text, Any] = None) -> None:
        """Construct a new tokenizer using the WhitespaceTokenizer framework."""
        super().__init__(component_config)
        self.lang = self.component_config["lang"]
        print(f".. tokenizer with lang {self.lang}")

    def tokenize(
            self, text: Text, attribute: Text = TEXT_ATTRIBUTE
    ) -> List[Token]:
        if self.lang in ('zh', 'ja'):
            r = query_data_by_url(cf.servant_by_lang(self.lang), 'tokens', {'lang': self.lang, 'sents': text})
            words = r['data']
            running_offset = 0
            tokens = []
            for word in words:
                word_offset = text.index(word, running_offset)
                word_len = len(word)
                running_offset = word_offset + word_len
                tokens.append(Token(word, word_offset))
            return tokens
        return super().tokenize(text, attribute)





import streamlit as st
from durable.engine import Host

from interacts.common import display_lang_selector
from interacts.sl_utils import all_labels, write_styles
from interacts.tracker_streamlit import enable_streamlit_tracker

from durable.lang import *
import durable.lang as dlang

enable_streamlit_tracker()
write_styles()

with ruleset('wordnet'):
    @when_all(m.subject.matches('.*thing.*, entity.*'))
    def matches(c):
        st.write('string matches pat -> {0}'.format(c.m.subject))


    @when_all(m.amod.anyItem(item.matches('.*thing.*, entity.*')))
    def amod_with(c):
        st.write('amod pat -> {0}'.format(c.m.amod))


    @when_all(m.subject.imatches('.*thing.*'))
    def contains(c):
        st.write('string contains pat -> {0}'.format(c.m.subject))

def sidebar():
    cur_lang=display_lang_selector()

def rule_test():
    host = Host()
    ruleset_definitions = {}
    for name, rset in dlang._rulesets.items():
        full_name, ruleset_definition = rset.define()
        ruleset_definitions[full_name] = ruleset_definition

    host.register_rulesets(ruleset_definitions)
    # assert_fact
    host.post('wordnet', {'subject': 'main.n.01, body_of_water.n.01, thing.n.12, physical_entity.n.01, entity.n.01'})
    host.post('wordnet', {'amod': ['main.n.01, body_of_water.n.01, thing.n.12, physical_entity.n.01, entity.n.01']})

def main():
    sidebar()
    st.subheader("Application")
    rule_test()

if __name__ == '__main__':
    main()




#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2000 - 2003 Charles W. Rapp.
# All Rights Reserved.
#
# Contributor(s):
#       Port to Python by Francois Perrad, francois.perrad@gadz.org
#
# Function
#   Main
#
# Description
#  This routine starts the finite state machine running.
#
# RCS ID
# $Id$
#
# CHANGE LOG
# $Log$
# Revision 1.2  2009/04/19 14:39:48  cwrapp
# Added call to enterStartState before issuing first FSM transition.
#
# Revision 1.1  2005/05/28 17:48:29  cwrapp
# Added Python examples 1 - 4 and 7.
#
#

from fsm.ex2 import AppClass_sm

class AppClass:

	def __init__(self):
		self._fsm = AppClass_sm.AppClass_sm(self)
		self._is_acceptable = False

		# Uncomment to see debug output.
		#self._fsm.setDebugFlag(True)

	def CheckString(self, string):
		self._fsm.enterStartState()
		for c in string:
			if c == '0':
				self._fsm.Zero()
			elif c == '1':
				self._fsm.One()
			else:
				self._fsm.Unknown()
		self._fsm.EOS()
		return self._is_acceptable

	def Acceptable(self):
		self._is_acceptable = True

	def Unacceptable(self):
		self._is_acceptable = False



from typing import Text, Dict, Sequence, List
import json
import json_utils
import io
import logging

logger = logging.getLogger(__name__)

class EntityData(object):
    def __init__(self, start, end, value, entity, norm):
        self.start=start
        self.end=end
        self.value=value
        self.entity=entity
        self.norm=norm
        # print('..', self.norm)
        
    @property
    def json(self):
        return {
            "start":self.start,
            "end":self.end,
            "value":self.value if self.norm=='o' else self.norm,
            "entity":self.entity
        }
    
class FactData(object):
    def __init__(self, text, intent):
        self.text=text
        self.intent=intent
        self.entities=[]  # EntityData

    def collect_synonyms(self):
        return [(x.norm, x.value) for x in self.entities if x.norm!='o']
        
    @property
    def json(self):
        return {
            "text":self.text,
            "intent":self.intent,
            "entities":[u.json for u in self.entities]
        }
    
class NluData(object):
    def __init__(self):
        self.examples=[]  # FactData

    def group_synonyms(self):
        import itertools
        import operator
        all_norms=[]
        for l in self.examples:
            all_norms.extend(l.collect_synonyms())
        it = itertools.groupby(all_norms, operator.itemgetter(0))
        for key, subiter in it:
            yield {"value":key, "synonyms": [item[1] for item in subiter]}

    @property
    def json(self):
        return {
          "rasa_nlu_data": {
            "common_examples": [u.json for u in self.examples],
            'entity_synonyms': list(self.group_synonyms())
          }
        }

def parse(dataset:NluData, intent:Text, sentence:Text):
    from saai.dataset.intent_dataset import IntentUtterance, SlotChunk
    # from snips_nlu.dataset.intent import IntentUtterance, SlotChunk
    u = IntentUtterance.parse(sentence)
    fact=FactData(u.input, intent)
    # fact = FactData(u.text, intent)
    for chunk in u.chunks:    
        if type(chunk)==SlotChunk:
            logger.debug("{}: {}-{}".format(chunk.text,
                chunk.range.start,
                chunk.range.end))
            entity=EntityData(chunk.range.start, chunk.range.end,
                              chunk.text, chunk.name, chunk.entity)
            fact.entities.append(entity)
    dataset.examples.append(fact)

class CorpusProcs(object):
    ## ./corpus_procs.py view_json_data "json/demo-rasa_zh.json"
    def view_json_data(self, file_name):
        data=json_utils.read_json_file(file_name)
        index=1
        for item in data["rasa_nlu_data"]["common_examples"]:
            print("{}. {} ({})".format(index, item["text"], item["intent"]))
            children=item['entities']
            if len(children)>0:
                for c in children:
                    print("\t{}-{}: {}({})".format(c["start"], c["end"], c["value"], c["entity"]))
            index=index+1

    def check_json(self, input_file):
        data=json_utils.read_json_file(input_file)
        dataset=NluData()
        for item in data["rasa_nlu_data"]["common_examples"]:
            # print("{}. {} ({})".format(index, item["text"], item["intent"]))
            fact=FactData(item["text"], item["intent"])
            children=item['entities']
            if len(children)>0:
                for c in children:
                    # print("\t{}-{}: {}({})".format(c["start"], c["end"], c["value"], c["entity"]))
                    entity=EntityData(c["start"], c["end"], c["value"], c["entity"], c["value"])
                    fact.entities.append(entity)
            dataset.examples.append(fact)
        print(json.dumps(dataset.json, indent=2, sort_keys=False, ensure_ascii=False))

    ## ./corpus_procs.py gen_dataset corpus/cn-Samples.md json/cn-Samples.json
    def gen_dataset(self, filename, output):
        with open(filename) as f:
            lines=f.readlines()

        dataset=NluData()
        intent=""
        for line in lines:
            line_s=line.strip()
            if line.startswith("##"):        
                intent=line[3:].strip()
                logger.debug(intent)
            elif len(line_s)==0 or line_s.startswith("#"):
                pass
            else:
                parse(dataset, intent, line_s)


        result=(json.dumps(dataset.json, indent=2, sort_keys=False, ensure_ascii=False))
        with io.open(output, 'w', encoding="utf-8") as f:
            f.write(result)

        logger.info(f".. write to {output} ok.")

    def gen_datasets(self, lang_prefix='cn', target_prefix='/pi/ws/sagas-ai/nlu_multilang/zh/'):
        import glob
        import os
        prefix='/pi/ws/sagas-ai/nlu_multilang'
        trans_files = [(x, target_prefix + os.path.basename(x).replace('.md', '.json')) for x in
                       glob.glob(f'{prefix}/corpus/{lang_prefix}-*.md')]
        for f in trans_files:
            self.gen_dataset(f[0], f[1])

    def gen_local(self):
        import glob
        import os

        target_prefix='./data/'
        prefix='./corpus'
        trans_files = [(x, target_prefix + os.path.basename(x).replace('.md', '.json')) for x in
                       glob.glob(f'{prefix}/*.md')]
        for f in trans_files:
            print(f'.. generate {f[0]} -> {f[1]}')
            self.gen_dataset(f[0], f[1])

if __name__ == '__main__':
    import fire
    fire.Fire(CorpusProcs)




#!/usr/bin/env python

#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2000 - 2003 Charles W. Rapp.
# All Rights Reserved.
#
# Contributor(s):
#       Port to Python by Francois Perrad, francois.perrad@gadz.org
#
# Function
#   Main
#
# Description
#  This routine starts the finite state machine running.
#
# RCS ID
# $Id$
#
# CHANGE LOG
# $Log$
# Revision 1.2  2014/07/12 08:20:57  fperrad
# compat python3
#
# Revision 1.1  2005/05/28 17:48:29  cwrapp
# Added Python examples 1 - 4 and 7.
#
#

import sys

import AppClass

retcode = 0
if len(sys.argv) < 2:
	print("No string to check.\n")
	retcode = 2
elif len(sys.argv) > 2:
	print("Only one argument is accepted.\n")
	retcode = 3
else:
	appobject = AppClass.AppClass()
	str = sys.argv[1]
	if appobject.CheckString(str) == False:
		result = "not acceptable"
		retcode = 1
	else:
		result = "acceptable"
	print('The string "%s" is %s.\n' % (str, result))
sys.exit(retcode)



from aiohttp import web
import textwrap
import asyncio
import functools
import json

import logging

logger = logging.getLogger(__name__)

def ensure_paras(names, paras):
    for name in names:
        if name not in paras:
            return False
    return True

class AgentServ(object):
    def __init__(self, conf_file):
        from saai.agent_procs import BotsConf
        self.conf_file=conf_file
        self.conf=BotsConf(conf=self.conf_file)

    async def intro(self, request):
        txt = textwrap.dedent("""\
            Type {url}/hello/John  {url}/simple or {url}/change_body
            in browser url bar
        """).format(url='127.0.0.1:18099')
        binary = txt.encode('utf8')
        resp = web.StreamResponse()
        resp.content_length = len(binary)
        resp.content_type = 'text/plain'
        await resp.prepare(request)
        await resp.write(binary)
        return resp

    async def post(self, request):
        """
        POST - http://localhost:18099/post/path
        body(json) - {"x":5}
        :param request:
        :return:
        """
        data = await request.json()
        info=request.match_info.get('info', '_')
        print(data, type(data))
        return web.json_response({
            'method': 'post',
            'info': info,
            # 'args': dict(request.GET),
            'data': dict(data),
            'headers': dict(request.headers),
        }, dumps=functools.partial(json.dumps, indent=4))

    async def handle_message(self, request):
        """
        POST - http://localhost:18099/message/sender_id
        body(json) - {"x":5}
        :param request:
        :return:
        """
        from saai.agent_procs import BotsConf, handle_message

        data = await request.json()
        sender=request.match_info.get('sender', '_')
        print(data, type(data))
        if not ensure_paras(('mod', 'sents', 'lang'), data):
            # 400 Bad Request
            return web.json_response({
                'error':'invalidate parameters',
                'method': 'handle_message',
                'sender': sender,
                'data': dict(data),
                'headers': dict(request.headers),
            }, status=400, dumps=functools.partial(json.dumps, indent=4))

        # invoke agent
        # handle_message(bot, text, sender='default')
        resp=await handle_message(data['mod'], data['sents'], sender=sender, conf=self.conf)
        return web.json_response(resp, dumps=functools.partial(json.dumps, indent=4))

    async def handle_reload(self, request):
        from saai.agent_procs import get_agent
        data = await request.json()
        # ignore parameter 'model'
        if not ensure_paras(['mod'], data):
            # 400 Bad Request
            return web.json_response({
                'error': 'invalidate parameters',
                'method': 'handle_reload',
                'data': dict(data),
                'headers': dict(request.headers),
            }, status=400, dumps=functools.partial(json.dumps, indent=4))

        mod = data['mod']
        await get_agent(mod, self.conf, force=True)
        resp={mod:'ok'}
        return web.json_response(resp, dumps=functools.partial(json.dumps, indent=4))

    # Notice: 在servant模式下训练nlu模块是有问题的(rasa-1.6), 所以目前不要使用在线训练nlu,
    # 而是通过命令行形式来进行, 如: $ python -m saai.nlu_mod_procs train en
    async def handle_reload_nlu(self, request):
        from saai.nlu_mod_procs import nlu_mods

        data = await request.json()
        # ignore parameter 'model'
        if not ensure_paras(['mod'], data):
            # 400 Bad Request
            return web.json_response({
                'error': 'invalidate parameters',
                'method': 'handle_reload_nlu',
                'data': dict(data),
                'headers': dict(request.headers),
            }, status=400, dumps=functools.partial(json.dumps, indent=4))

        mod = data['mod']
        result=nlu_mods.reload(mod)
        resp={mod:result}
        return web.json_response(resp, dumps=functools.partial(json.dumps, indent=4))

    async def handle_parse(self, request):
        from saai.nlu_mod_procs import nlu_mods
        data = await request.json()
        # ignore parameter 'model'
        if not ensure_paras(('project', 'q'), data):
            # 400 Bad Request
            return web.json_response({
                'error':'invalidate parameters',
                'method': 'handle_parse',
                'data': dict(data),
                'headers': dict(request.headers),
            }, status=400, dumps=functools.partial(json.dumps, indent=4))

        lang=data['project']
        sents=data['q']

        resp =await nlu_mods.parse_async(sents, lang)
        return web.json_response(resp, dumps=functools.partial(json.dumps, indent=4))

    def init(self, loop):
        self.loop = loop
        app = web.Application()
        app.router.add_get('/', self.intro)
        app.router.add_post('/post/{info}', self.post)
        app.router.add_post('/message/{sender}', self.handle_message)
        app.router.add_post('/parse', self.handle_parse)
        app.router.add_post('/reload', self.handle_reload)
        app.router.add_post('/reload_nlu', self.handle_reload_nlu)

        return app

class ServantRunner(object):
    def app(self, conf='agents'):
        """
        $ python -m saai.agent_servant app
        """
        # conf_prefix='/pi/ws/sagas-ai/saai/conf'
        if not conf.startswith('/'):
            conf=f"{conf}.json"
        serv = AgentServ(conf)
        v_loop = asyncio.get_event_loop()
        v_loop.set_debug(True)
        web.run_app(serv.init(v_loop), port=18099)

if __name__ == '__main__':
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(ServantRunner)





import streamlit as st

from interacts.common import display_lang_selector
from interacts.sl_utils import all_labels, write_styles
from interacts.tracker_streamlit import enable_streamlit_tracker

from sagas.conf.conf import cf
from sagas.nlu.ruleset_procs import parse_sents, equals, group_by, children
from sagas.nlu.uni_remote_viz import list_contrast, display_doc_deps

enable_streamlit_tracker()
write_styles()

def build_token(sent, word, dc, domains):
    token = {**word.ctx, **group_by(domains)}
    # add_head(domains, dc, sent)
    if dc.governor != 0:
        head = sent.words[dc.governor - 1]
        token['head'] = head.ctx
    token['dc'] = dc.ctx
    return token

def get_aux_domain(sent):
    rs = []
    for word in filter(lambda w: w.upos == "AUX", sent.words):
        # dc=sent.words[word.governor-1]
        if word.governor == 0:
            # if the aux word is root; (这种情形会出现在德语依存分析中, 但在英语依存分析中是正常的)
            dc = word
            delegator = True
        else:
            dc = sent.words[word.governor - 1]
            delegator = False
        # print('℗', word.text, word.dependency_relation, word.governor, '☇' , dc.text)
        # print('\t', dc.index, dc.text, get_children_list(sent, dc))
        domains = []
        # stems = []
        # 需要收集的是aux单词依赖的对象的关联集, 而不是aux单词自身的关联集
        for c in filter(lambda w: equals(w.governor, dc.index), sent.words):
            # print('\t', c.index, c.text, get_children_list(sent, c))
            # domains.append((c.dependency_relation, c.index, c.text, c.lemma,
            #                 get_children_list(sent, c), get_word_features(c)))
            # add_domain(domains, stems, c, sent)
            c_domains = [w.ctx for w in children(c, sent)]
            domains.append({**c.ctx, **group_by(c_domains)})

        token=build_token(sent, word, dc, domains)
        rs.append(token)
        # rs.append({'type':'aux_domains', 'word': word.text, 'lemma':word.lemma,
        #           'rel': word.dependency_relation, 'governor': word.governor, 'head': dc.text,
        #           'head_pos': dc.upos.lower(), 'delegator':delegator,
        #           'index': word.index, 'domains': domains, 'stems':stems})
    return rs


def get_subj_domain(sent):
    rs = []
    for word in filter(lambda w: w.dependency_relation.endswith('subj'), sent.words):
        dc = sent.words[word.governor - 1]
        # print('℗', word.text, word.dependency_relation, word.governor, '☇' , dc.text)
        domains = []
        stems = []
        # 需要收集的是subj依赖的对象的关联集
        for c in filter(lambda w: equals(w.governor, dc.index), sent.words):
            # print('\t', c.index, c.text, get_children_list(sent, c))
            # domains.append((c.dependency_relation, c.index, c.text, c.lemma,
            #                 get_children_list(sent, c), get_word_features(c)))
            # add_domain(domains, stems, c, sent)
            c_domains = [w.ctx for w in children(c, sent)]
            domains.append({**c.ctx, **group_by(c_domains)})

        # add_head(domains, dc, sent)
        token = build_token(sent, word, dc, domains)
        rs.append(token)
        # rs.append({'type':'subj_domains', 'word': word.text, 'lemma':word.lemma,
        #           'rel': word.dependency_relation, 'governor': word.governor, 'head': dc.text,
        #           'head_pos': dc.upos.lower(), 'head_feats':[dc.lemma, dc.upos, dc.xpos],
        #           'index': word.index, 'domains': domains, 'stems':stems})
    return rs


class OperValue(object):

    def __init__(self, vtype=None, left=None, op=None, right=None, alias=None):
        self.alias = alias
        self._type = vtype
        self._left = left
        self._op = op
        self._right = right

    def __lt__(self, other):
        return OperValue(self._type, self._left, '$lt', other, self.alias)

    # //	__floordiv__(self, other)
    def __floordiv__(self, other):
        return OperValue(self._type, self, '$sub', other, self.alias)

    def __rshift__(self, other):
        return OperValue(self._type, self, '$for', other, self.alias)

    #     def __getattr__(self, name):
    #         if self._type:
    #             if self._left:
    #                 return value(self._type, '{0}.{1}'.format(self._left, name))
    #             else:
    #                 return value(self._type, name)
    #         else:
    #             return value(self.alias, name)
    def __call__(self, *args):
        return OperValue(self._type, self._left, op='predicate', right=args, alias=self.alias)


class Closure(object):
    def __getattr__(self, name):
        return OperValue(name)

def desc_expr(el, domains, lang):
    from sagas.nlu.inspector_wordnet import predicate

    if el._type.startswith('__'):
        comp='__'
        sub = el._type
        oper=el._type[2:]
    elif '_' in el._type:
        parts=el._type.split('_')
        comp=f"{parts[0]}'s {parts[1]}"
        sub=parts[0]
        oper=parts[1]
    else:
        comp=f"{el._type}'s lemma"
        sub=el._type
        oper='equals'
    val=', '.join(el._right)
    cond=eval_path(domains, sub)
    eq=lambda vals: any([v in vals for v in cond.split('/')])
    oper_mappings={'equals': eq,
                   'text': eq,
                   'cat': lambda vals: any([predicate(e, cond, lang, '*') for e in vals])
                   }
    if oper in oper_mappings:
        result=oper_mappings[oper](el._right)
        msg=''
    else:
        result=False
        msg=f'invalid operator {oper}'
    st.markdown(f"- `{comp}` {el._op} `{val}`, *{cond}* -> {result} {msg}")

    return result, msg

def eval_path(domains, path):
    from jsonpath_ng import jsonpath, parse

    if path.startswith('__'):
        return domains['lemma']+'/'+domains['text']
    else:
        prefix='$.'
        suffix='.text,lemma'
        parts=path.split('/')
        parts_str='.'.join([f"{t}[*]" for t in parts])
        expr= f"{prefix}{parts_str}{suffix}"
        parser=parse(expr)
        word = '/'.join([match.value for match in parser.find(domains)])
        return word

def predicate(domains, val, lang):
    results=[]
    st.markdown(f"*{val._left._type}* {val._left._op} {val._left._right}")
    results.append(desc_expr(val._left, domains, lang))

    # desc_expr(val._left)
    expr=val._right
    if isinstance(expr, list):
        for el in expr:
            results.append(desc_expr(el, domains, lang))
    else:
        results.append(desc_expr(expr, domains, lang))
    return results

def sure(doc, val):
    st.write('-' * 10)
    st.write(val._left._type, val._op, val._right._type)
    st.write(val._left._op, val._left._right, val._right._op, val._right._right)

ud = Closure()
def testing(doc, lang):
    # m = value('$m')
    # s = value('$s')
    # aux = value('$aux')
    # nsubj = value('$nsubj')

    # print(m.first._left)
    # st.write(aux._left, aux._type)
    # sure(aux('is') // nsubj('animal'))
    for el in doc:
        st.markdown(f"`{el['lemma']}` >> *{el['dc']['lemma']}*")
        r1=predicate(el, ud.__text('will') >> [ud.nsubj('what'), ud.dc_cat('weather')], lang)
        # r2=predicate(el, ud.__cat('be') >> [ud.nsubj('what'), ud.dc_cat('animal/object')], lang)
        st.write([r[0] for r in r1], all([r[0] for r in r1]))

    # sure(doc, ud.aux('is') // ud.nsubj('animal'))
    sure(doc, ud.aux('is') >> ud.nsubj('animal'))

def sidebar():
    cur_lang=display_lang_selector()

def parse_aux():
    lang = 'en'
    sents = 'what will be the weather in three days?'
    st.write(sents)
    data = {'lang': lang, "sents": sents, 'engine': cf.engine(lang)}
    doc_jsonify, resp = parse_sents(data)

    domains = get_aux_domain(doc_jsonify)
    testing(domains, 'en')

    # show analyse graph
    gv = display_doc_deps(doc_jsonify, resp)
    st.graphviz_chart(gv)

    st.write(domains)

def parse_subj():
    lang = 'ru'
    sents = 'Яблоко - это здоровый фрукт.'
    st.write(sents)
    data = {'lang': lang, "sents": sents, 'engine': cf.engine(lang)}
    doc_jsonify, resp = parse_sents(data)

    domains = get_subj_domain(doc_jsonify)
    testing(domains, 'ru')

    gv=display_doc_deps(doc_jsonify, resp)
    st.graphviz_chart(gv)

    st.write(domains)

def main():
    sidebar()
    st.subheader("Application")
    opt=st.sidebar.selectbox('Which fixture', ['parse subj', 'parse aux'])
    if opt=='parse subj':
        parse_subj()
    elif opt=='parse aux':
        parse_aux()

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- tab-width: 4; -*-

#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2000 - 2003 Charles W. Rapp.
# All Rights Reserved.
#
# Contributor(s):
#       Port to Python by Francois Perrad, francois.perrad@gadz.org
#
# traffic --
#
#  Use state machines to do a very simple simulation of stoplights.
#
# RCS ID
# $Id$
#
# CHANGE LOG
# $Log$
# Revision 1.2  2008/04/23 12:54:03  fperrad
# + fix #1934497 : remove -w in shebang
#
# Revision 1.1  2005/05/28 17:48:30  cwrapp
# Added Python examples 1 - 4 and 7.
#
#

# Don't run this script on macos
from Tkinter import *

# Load in the stoplight and vehicles classes.
from fsm.ex4 import Stoplight
from fsm.ex4 import Vehicle

class Top:

	# DisplaySliders --
	#
	#   Display the window which contains the sliders for dynamically
	#   configuring the traffic demo.
	#
	# Arguments:
	#   None.

	def DisplaySliders(self):
		# Immediatly disable the window to prevent it from being
		# selected again.
		self._ConfigButton.configure(state=DISABLED)

		# Put the sliders in a separate window. Create three frames,
		# one for each kind of slider.
		SliderFrame = Toplevel(self._root)
		SliderFrame.title("Traffic Configuration")

		# Put in the slider controls for setting the traffic light times
		# (how long each light stays green or yellow), how often new
		# vehicles appear and how fast vehicles move.
		Scale(SliderFrame,
				from_=5,
				to=20,
				variable=self._NSGreenTime,
				label="North/South green light timer (in seconds)",
				orient=HORIZONTAL,
				tickinterval=5,
				showvalue=None,
				sliderrelief=SUNKEN,
				length=250,
				command=lambda x:self._Stoplight.setLightTimer("NSGreenTimer", int(x)),
		).pack(
				side=TOP,
		)
		Scale(SliderFrame,
				from_=5,
				to=20,
				variable=self._EWGreenTime,
				label="East/West green light timer (in seconds)",
				orient=HORIZONTAL,
				tickinterval=5,
				showvalue=None,
				sliderrelief=SUNKEN,
				length=250,
				command=lambda x:self._Stoplight.setLightTimer("EWGreenTimer", int(x)),
		).pack(
				side=TOP,
		)
		Scale(SliderFrame,
				from_=2,
				to=8,
				variable=self._YellowTime,
				label="Yellow light timer (in seconds)",
				orient=HORIZONTAL,
				tickinterval=2,
				showvalue=None,
				sliderrelief=SUNKEN,
				length=250,
				command=lambda x:self._Stoplight.setLightTimer("YellowTimer", int(x)),
		).pack(
				side=TOP,
		)

		def setAppearanceRate(rate):
			self._AppearanceTimeout = int(rate) * 1000

		Scale(SliderFrame,
				from_=5,
				to=15,
				variable=self._AppearanceRate,
				label="Vehicle appearance rate (in seconds)",
				orient=HORIZONTAL,
				tickinterval=2,
				showvalue=None,
				sliderrelief=SUNKEN,
				length=250,
				command=setAppearanceRate,
		).pack(
				side=TOP,
		)
		Scale(SliderFrame,
				from_=1,
				to=5,
				variable=self._VehicleSpeed,
				label="Vehicle speed (in seconds)",
				orient=HORIZONTAL,
				tickinterval=1,
				showvalue=None,
				sliderrelief=SUNKEN,
				length=250,
				command=lambda x:Vehicle.Vehicle.setSpeed(int(x)),
		).pack(
				side=TOP,
		)

		# When the slider window is destroyed, re-enable the
		# configure button.
		SliderFrame.bind('<Destroy>', lambda x:self._ConfigButton.configure(state=NORMAL))

		SliderFrame.focus()

	# StartDemo --
	#
	#   Start the traffic flowing. Do this by having the
	#   stoplight and vehicle objects start their timers.
	#   Also start the "make vehicles" timer.
	#
	# Arguments:
	#   None.

	def StartDemo(self):
		self._Stoplight.Start()

		# Create four vehicles, one for each direction.
		self.makeVehicles()

		# Every minute, go through the vehicle list and
		# delete those vehicles that have completed their
		# trip.
		self._CollectTimerID = self._root.after(60000, self.garbageCollect)

		# Disable the start button and enable the pause and stop button.
		self._StartButton.configure(state=DISABLED)
		self._PauseButton.configure(state=NORMAL)
		self._StopButton.configure(state=NORMAL)

	# PauseDemo --
	#
	#   Temporarily pause this demo.
	#
	# Arguments:
	#   None.

	def PauseDemo(self):
		self._PauseFlag = True

		# Tell the stop light and vehicles to temporarily
		# stop their timers.
		self._Stoplight.Pause()

		for vehicle in self._VehicleList:
			vehicle.Pause()

		# Stop the vehicle deletion timer.
		if self._CollectTimerID >= 0:
			self._root.after_cancel(self._CollectTimerID)
			self._CollectTimerID = -1

		# Disable the pause button and enable the continue button.
		self._PauseButton.configure(state=DISABLED)
		self._ContinueButton.configure(state=NORMAL)

	# ContinueDemo --
	#
	#   Pick up the demo where you left off.
	#
	# Arguments:
	#   None.

	def ContinueDemo(self):
		self._PauseFlag = False

		# If the vehicle appearance timer expired during the pause,
		# then make some vehicles now.
		if self._AppearanceTimerID == -2:
			self.makeVehicles()

		# Tell the stop light and vehicles to temporarily
		# stop their timers.
		self._Stoplight.Continue()

		for vehicle in self._VehicleList:
			vehicle.Continue()

		# Enable the pause button and disable the continue button.
		self._PauseButton.configure(state=NORMAL)
		self._ContinueButton.configure(state=DISABLED)

	# StopDemo --
	#
	#   Stop the demo and delete all vehicles.
	#
	# Arguments:
	#   None.

	def StopDemo(self):
		self._Stoplight.Stop()

		for vehicle in self._VehicleList:
			vehicle.Stop()
		self._VehicleList = []

		if self._AppearanceTimerID >= 0:
			self._root.after_cancel(self._AppearanceTimerID)
			self._AppearanceTimerID = -1

		if self._CollectTimerID >= 0:
			self._root.after_cancel(self._CollectTimerID)
			self._CollectTimerID = -1

		# Enable the start button and disable all others.
		self._StartButton.configure(state=NORMAL)
		self._PauseButton.configure(state=DISABLED)
		self._ContinueButton.configure(state=DISABLED)
		self._StopButton.configure(state=DISABLED)

	# makeVehicles --
	#
	#   Create four new vehicles to move on the map. When
	#   done, set a timer to make even more later.
	#
	# Arguments:
	#   None.

	def makeVehicles(self):
		self._AppearanceTimerID = -1

		# Don't make vehicles if we are paused. Just remember that
		# the timer expired and call this routine when the demo is
		# continued.
		if self._PauseFlag:
			self._AppearanceTimerID = -2
		else:
			for direction in ('north', 'south', 'east', 'west'):
				vehicle = Vehicle.Vehicle(self._Stoplight, direction, self._Canvas)
				self._VehicleList.append(vehicle)

			# Gentlemen, start your engines.
			for vehicle in self._VehicleList:
				vehicle.Start()

			self._AppearanceTimerID = self._root.after(self._AppearanceTimeout, self.makeVehicles)

	# garbageCollect --
	#
	#   Delete those vehicles that have completed their trip.
	#
	# Arguments:
	#   None.
	#

	def garbageCollect(self):
		self._CollectTimerID = -1

		NewVehicleList = []
		for vehicle in self._VehicleList:
			if vehicle.isDone():
				vehicle.Delete()
			else:
				NewVehicleList.append(vehicle)
		self._VehicleList = NewVehicleList

		# Reset this timer.
		self._CollectTimerID = self._root.after(60000, self.garbageCollect)

	def __init__(self, root):
		self._root = root
		# Default settings.
		self. _VehicleList = []
		self._AppearanceTimerID = -1
		self._AppearanceTimeout = 8000
		self._CollectTimerID = -1
		self._NSGreenTime = IntVar()
		self._NSGreenTime.set(7)
		self._EWGreenTime = IntVar()
		self._EWGreenTime.set(5)
		self._YellowTime = IntVar()
		self._YellowTime.set(2)
		self._AppearanceRate = IntVar()
		self._AppearanceRate.set(8)
		self._VehicleSpeed = IntVar()
		self._VehicleSpeed.set(2)
		self._PauseFlag = False

		# Set up the window in which the stop light demo will appear.
		# Also create two other frames. One will hold the sliders for
		# dynamically configuring the demo and the other buttons to
		# start, pause, continue and quit the demo.
		root.title("Stoplight demo")

		ConfigFrame = Frame(root,
				borderwidth=4,
				relief=FLAT,
				height=15,
				width=250,
		)
		ConfigFrame.pack(
				side=TOP,
				fill=BOTH,
		)
		MainFrame = Frame(root,
				borderwidth=4,
				relief=FLAT,
				height=250,
				width=250,
		)
		MainFrame.pack(
				side=TOP,
				fill=BOTH,
		)
		ButtonFrame = Frame(root,
				borderwidth=4,
				relief=FLAT,
				height=15,
				width=250,
		)
		ButtonFrame.pack(
				side=TOP,
				fill=BOTH,
		)

		# Put a single button in the configure frame which causes the
		# slider window to pop up.
		self._ConfigButton = Button(ConfigFrame,
				text="Configure...",
				command=self.DisplaySliders,
		)
		self._ConfigButton.pack(
				side=RIGHT,
		)

		# Create a canvas in which the stop light graphics will appear.
		self._Canvas = Canvas(MainFrame,
				borderwidth=2,
				background='white',
				relief=RAISED,
				height=250,
				width=250,
		)
		self._Canvas.pack(
				side=TOP,
				fill=BOTH,
		)

		# Create the stoplight and specify which direction initially has
		# the green light.
		self._Stoplight = Stoplight.Stoplight(self._Canvas)

		# Add a button which allows the demo to be started, paused, continued
		# and stopped.
		self._StartButton = Button(ButtonFrame,
				text="Start",
				command=self.StartDemo,
		)
		self._StartButton.pack(
				side=LEFT,
		)
		self._PauseButton = Button(ButtonFrame,
				text="Pause",
				state=DISABLED,
				command=self.PauseDemo,
		)
		self._PauseButton.pack(
				side=LEFT,
		)
		self._ContinueButton = Button(ButtonFrame,
				text="Continue",
				state=DISABLED,
				command=self.ContinueDemo,
		)
		self._ContinueButton.pack(
				side=LEFT,
		)
		self._StopButton = Button(ButtonFrame,
				text="Stop",
				state=DISABLED,
				command=self.StopDemo,
		)
		self._StopButton.pack(
				side=LEFT,
		)

		# Cntl-C stops the demo.
		root.bind('<Control-c>', lambda x: sys.exit(0))


root = Tk()
top = Top(root)
root.mainloop()



from __future__ import print_function, absolute_import

from abc import ABCMeta, abstractmethod
from builtins import object
from pathlib import Path

from future.utils import with_metaclass

from saai.dataset.constants import UTTERANCES, SLOT_NAME, ENTITY, TEXT, DATA

INTENT_FORMATTING_ERROR = AssertionError(
    "Intent file is not properly formatted")


class IntentDataset(object):
    """Dataset of an intent

    Can parse utterances from a text file or an iterator.

    An example of utterance is:

        "the [role:role](president) of [country:country](France)"

    a Tag is in this format:

        [slot:entity_name](text_to_tag)

    Attributes:
        intent_name (str): name of the intent
        utterances (list of :class:`.IntentUtterance`): intent utterances
    """

    def __init__(self, intent_name):
        self.intent_name = intent_name
        self.utterances = []

    @classmethod
    def from_file(cls, filepath):
        filepath = Path(filepath)
        stem = filepath.stem
        if not stem.startswith("intent_"):
            raise AssertionError("Intent filename should start with 'intent_' "
                                 "but found: %s" % stem)
        intent_name = stem[7:]
        if not intent_name:
            raise AssertionError("Intent name must not be empty")
        with filepath.open(encoding="utf-8") as f:
            lines = iter(l.strip() for l in f if l.strip())
            return cls.from_iter(intent_name, lines)

    @classmethod
    def from_iter(cls, intent_name, samples_iter):
        """Generates a dataset from an iterator of samples"""
        dataset = cls(intent_name)
        for sample in samples_iter:
            utterance = IntentUtterance.parse(sample)
            dataset.add(utterance)
        return dataset

    def add(self, utterance):
        """Adds an :class:`.IntentUtterance` to the dataset"""
        self.utterances.append(utterance)

    @property
    def json(self):
        """Intent dataset in json format"""
        return {
            UTTERANCES: [
                {DATA: [chunk.json for chunk in utterance.chunks]}
                for utterance in self.utterances
            ]
        }

    @property
    def entities_names(self):
        """Set of entity names present in the intent dataset"""
        return set(chunk.entity for u in self.utterances
                   for chunk in u.chunks if isinstance(chunk, SlotChunk))


class IntentUtterance(object):
    def __init__(self, input, chunks):
        self.input = input
        self.chunks = chunks

    @property
    def annotated(self):
        """Annotates with *

        Returns: The sentence annotated just with stars

        Examples:

            >>> from snips_nlu.cli.dataset.intent_dataset import \
                IntentUtterance
            >>> p = "the [role:role](president) of [country:country](France)"
            >>> u = IntentUtterance.parse(p)
            >>> u.annotated
            'the *president* of *France*'
        """
        binput = bytearray(self.input, 'utf-8')
        acc = 0
        star = ord('*')
        for chunk in self.chunks:
            if isinstance(chunk, SlotChunk):
                binput.insert(chunk.range.start + acc, star)
                binput.insert(chunk.range.end + acc + 1, star)
                acc += 2
        return binput.decode('utf-8')

    @staticmethod
    def stripped(input, chunks):
        acc = 0
        s = ''
        new_chunks = []
        for chunk in chunks:
            start = chunk.range.start
            end = chunk.range.end
            s += input[start:end]
            if isinstance(chunk, SlotChunk):
                acc += chunk.tag_range.size
                rng = Range(start - acc, end - acc)
                new_chunk = SlotChunk(chunk.name, chunk.entity, rng,
                                      chunk.text, chunk.tag_range)
                new_chunks.append(new_chunk)
                acc += 1
            else:
                rng = Range(start - acc, end - acc)
                new_chunks.append(TextChunk(chunk.text, rng))
        return s, new_chunks

    @staticmethod
    def parse(string):
        """Parses an utterance

        Args:
            string (str): an utterance in the class:`.Utterance` format

        Examples:

            >>> from snips_nlu.cli.dataset.intent_dataset import \
                IntentUtterance
            >>> u = IntentUtterance.\
                parse("president of [country:default](France)")
            >>> len(u.chunks)
            2
            >>> u.chunks[0].text
            'president of '
            >>> u.chunks[0].range.start
            0
            >>> u.chunks[0].range.end
            13
        """
        sm = SM(string)
        capture_text(sm)
        string, chunks = IntentUtterance.stripped(string, sm.chunks)
        return IntentUtterance(string, chunks)


class Chunk(with_metaclass(ABCMeta, object)):
    def __init__(self, text, range):
        self.text = text
        self.range = range

    @abstractmethod
    def json(self):
        pass


class SlotChunk(Chunk):
    def __init__(self, slot_name, entity, range, text, tag_range):
        super(SlotChunk, self).__init__(text, range)
        self.name = slot_name
        self.entity = entity
        self.tag_range = tag_range

    @property
    def json(self):
        return {
            TEXT: self.text,
            SLOT_NAME: self.name,
            ENTITY: self.entity,
        }


class TextChunk(Chunk):
    @property
    def json(self):
        return {
            TEXT: self.text
        }


class Range(object):
    def __init__(self, start, end=None):
        self.start = start
        self.end = end

    @property
    def size(self):
        return self.end - self.start + 1


class SM(object):
    """State Machine for parsing"""

    def __init__(self, input):
        self.input = input
        self.chunks = []
        self.current = 0

    def add_slot(self, slot_start, name, entity):
        """Adds a named slot

        Args:
            slot_start (int): position where the slot tag started
            name (str): slot name
            entity (str): entity name
        """
        tag_range = Range(slot_start - 1)
        chunk = SlotChunk(slot_name=name, entity=entity, range=None, text=None,
                          tag_range=tag_range)
        self.chunks.append(chunk)

    def add_text(self, text):
        """Adds a simple text chunk using the current position"""
        start = self.current
        end = start + len(text)
        chunk = TextChunk(text=text, range=Range(start=start, end=end))
        self.chunks.append(chunk)

    def add_tagged(self, text):
        """Adds text to the last slot"""
        if not self.chunks:
            raise AssertionError("Cannot add tagged text because chunks list "
                                 "is empty")
        chunk = self.chunks[-1]
        chunk.text = text
        chunk.tag_range.end = self.current - 1
        chunk.range = Range(start=self.current, end=self.current + len(text))

    def find(self, s):
        return self.input.find(s, self.current)

    def move(self, pos):
        """Moves the cursor of the state to position after given

        Args:
            pos (int): position to place the cursor just after
        """
        self.current = pos + 1

    def peek(self):
        return self[0]

    def read(self):
        c = self[0]
        self.current += 1
        return c

    def __getitem__(self, key):
        current = self.current
        if isinstance(key, int):
            return self.input[current + key]
        elif isinstance(key, slice):
            start = current + key.start if key.start else current
            return self.input[slice(start, key.stop, key.step)]
        else:
            raise TypeError("Bad key type: %s" % type(key))


def capture_text(state):
    next_pos = state.find('[')
    sub = state[:] if next_pos < 0 else state[:next_pos]
    if sub.strip():
        state.add_text(sub)
    if next_pos >= 0:
        state.move(next_pos)
        capture_slot(state)


def capture_slot(state):
    slot_start = state.current
    next_pos = state.find(':')
    if next_pos < 0:
        raise INTENT_FORMATTING_ERROR
    else:
        slot_name = state[:next_pos]
        state.move(next_pos)
        next_pos = state.find(']')
        if next_pos < 0:
            raise INTENT_FORMATTING_ERROR
        entity = state[:next_pos]
        state.move(next_pos)
        state.add_slot(slot_start, slot_name, entity)
        if state.read() != '(':
            raise INTENT_FORMATTING_ERROR
        capture_tagged(state)


def capture_tagged(state):
    next_pos = state.find(')')
    if next_pos < 1:
        raise INTENT_FORMATTING_ERROR
    else:
        tagged_text = state[:next_pos]
        state.add_tagged(tagged_text)
        state.move(next_pos)
        capture_text(state)



from typing import Any, Dict, List, Text

from rasa.nlu.extractors import EntityExtractor
from rasa.nlu.training_data import Message
from sagas.conf.conf import cf

class FacadeEntityExtractor(EntityExtractor):

    provides = ["entities"]
    defaults = {
        # by default all dimensions recognized
        # dimensions can be configured to contain an array of strings
        # with the names of the dimensions to filter for
        "dimensions": None,
        "route": 'spacy/en',
    }

    def __init__(self, component_config: Text = None) -> None:
        super().__init__(component_config)

    def process(self, message: Message, **kwargs: Any) -> None:
        # print('..... ner_facade .......')
        all_extracted = self.add_extractor_name(self.extract_entities(message.text))
        dimensions = self.component_config["dimensions"]
        extracted = FacadeEntityExtractor.filter_irrelevant_entities(
            all_extracted, dimensions
        )
        message.set(
            "entities", message.get("entities", []) + extracted,
            add_to_output=True
        )

    def extract_entities(self, text: Text) -> List[Dict[Text, Any]]:
        import requests
        data = {"sents": text}
        route=self.component_config["route"]
        response = requests.post(f'{cf.ensure("facade")}/ner/{route}', json=data)
        if response.status_code==200:
            ents=response.json()
            entities = [
                {
                    "entity": ent['entity'],
                    "value": ent['text'],
                    "start": ent['start'],
                    "confidence": None,
                    "end": ent['end'],
                }
                for ent in ents
            ]
            return entities
        else:
            print(f".. error code {response.status_code}")
            return []




# This files contains your custom actions which can be used to run
# custom Python code.
#
# See this guide on how to implement these action:
# https://rasa.com/docs/rasa/core/actions/#custom-actions/


# This is a simple example for a custom action which utters "Hello World!"

from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher


class ActionHelloWorld(Action):

    def name(self) -> Text:
        return "action_hello_world"

    def run(self, dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:

        dispatcher.utter_message(text="Hello World!")

        return []



import requests
import asyncio
from pprint import pprint

def testing_tokenizer(text, cls, lang='en'):
    from rasa.nlu.training_data import TrainingData, Message
    defaults = {
        # Flag to check whether to split intents
        "intent_tokenization_flag": False,
        # Symbol on which intent should be split
        "intent_split_symbol": "_",
        # text will be tokenized with case sensitive as default
        "case_sensitive": True,
        "lang": lang,
    }

    tok=cls(defaults)
    example = Message(text, {
            "intent": "wish",
            "entities": []})
    # tokenizer
    tok.process(example, x='.')
    for token in example.get("tokens"):
        print(token.text, token.offset)

class SaaiCli(object):
    def version(self):
        from saai.version import __version__
        print(f"saai version: {__version__}")

    def comps(self):
        """
        $ python -m saai.saai_cli comps
        $ python -m saai comps

        :return:
        """
        from rasa.nlu import config
        conf = config.load('saai/sample_configs/config_tokenizer.yml')
        # conf.for_component('DucklingHTTPExtractor')
        return conf.component_names

    def tokens(self, text, lang):
        """
        $ python -m saai.saai_cli tokens "在终端上输出单词的定义和继承链" zh
        :param text:
        :param lang:
        :return:
        """
        from saai.multilang_tokenizer import MultilangTokenizer
        print(testing_tokenizer(text, MultilangTokenizer, lang))

    def bot_message(self, lang='en'):
        """
        $ python -m saai.saai_cli bot_message
        :param lang:
        :return:
        """
        # sents = '/behave_purpose{"object_type": "restaurant"}'
        # data = {'mod': 'genesis', 'lang': lang, "sents": sents}
        text = '/behave_purpose{"object_type": "text", "sents":"%s"}' % "do you have any restaurants"
        data = {'mod': 'genesis', 'lang': lang, "sents": text}
        response = requests.post(f'http://localhost:18099/message/my', json=data)
        print('status code:', response.status_code)
        return response.json()

    def nlu_ner(self, route, sents):
        """
        $ python -m saai.saai_cli nlu_ner spacy/en "I was born in Beijing."
        $ python -m saai.saai_cli nlu_ner id "Jokowi pergi ke Singapura."
        :param route:
        :param sents:
        :return:
        """
        data={"sents":sents}
        response = requests.post(f'http://localhost:1700/ner/{route}', json=data)
        print('status code:', response.status_code)
        pprint(response.json())

    def bot_reload(self, bot='genesis'):
        """
        $ python -m saai.saai_cli bot_reload genesis
        :param bot:
        :return:
        """
        response = requests.post(f'http://localhost:18099/reload', json={'mod': bot})
        print('status code:', response.status_code)
        return response.json()

    def nlu_parse(self, sents, lang='en'):
        """ Nlu parse routines
        $ python -m saai.saai_cli nlu_parse "Shenzhen ist das Silicon Valley für Hardware-Firmen" de
        $ python -m saai.saai_cli nlu_parse '附近有什么好吃的' zh
        $ python -m saai.saai_cli nlu_parse '六安市公安局裕安分局平桥派出所接到辖区居民戴某报警' zh
        $ python -m saai.saai_cli nlu_parse '一直打喷嚏怎么办' zh
        $ python -m saai.saai_cli nlu_parse "I was born in Beijing." en
        $ python -m saai.saai_cli nlu_parse "Я хочу поехать в москву" ru
        $ python -m saai.saai_cli nlu_parse "Jokowi pergi ke Singapura." id

        :param sents:
        :param lang:
        :return:
        """
        from sagas.conf.conf import cf
        from sagas.nlu.rasa_procs import invoke_nlu
        import json
        import sagas.tracker_fn as tc

        endpoint = cf.ensure('nlu_multilang_servant')
        print('.. with endpoing', endpoint)
        result = invoke_nlu(endpoint, lang, "current", sents)
        tc.emp('yellow', result)
        if result != None and len(result)>0:
            print(json.dumps(result, indent=4, ensure_ascii=False))

            intent = result["intent"]
            print('%s -> %f' % (intent['name'], intent['confidence']))
            entities = result['entities']
            print('entities ->', [ent['entity'] for ent in entities])

    def nlu_reload(self, lang='en'):
        """
        $ python -m saai.saai_cli nlu_reload en
        # or train and reload in docker env
        $ docker exec -it sagas_agent_servant_1 python -m saai.saai_cli nlu_reload en

        :param bot:
        :return:
        """
        from saai.nlu_mod_procs import train_mod
        print('.. training')
        train_mod(lang)
        print('.. reloading')
        response = requests.post(f'http://localhost:18099/reload_nlu', json={'mod': lang})
        print('status code:', response.status_code)
        return response.json()

    def multilang_nlu(self, lang, text):
        """
        $ python -m saai.saai_cli multilang_nlu en 'hi'
        $ python -m saai.saai_cli multilang_nlu en "I was born in Beijing."
        $ python -m saai.saai_cli multilang_nlu en "I was born in the spring of 1982."
        $ python -m saai.saai_cli multilang_nlu en "tomorrow at eight"
        $ python -m saai.saai_cli multilang_nlu ja "太郎は5月18日の朝9時に花子に会いに行った"
        $ python -m saai.saai_cli multilang_nlu ja "お皿を二枚ください。"

        :param lang:
        :param text:
        :return:
        """
        from saai.multi_nlu_client import multi_nlu
        from pprint import pprint

        loop = asyncio.get_event_loop()
        r= loop.run_until_complete(multi_nlu.multilang(lang, text))
        pprint(r)

if __name__ == '__main__':
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(SaaiCli)



import streamlit as st

from interacts.common import display_lang_selector
from interacts.sl_utils import all_labels, write_styles
from interacts.tracker_streamlit import enable_streamlit_tracker

from saai.ner_jieba import extract_entities
from sl_utils import HTML_WRAPPER

enable_streamlit_tracker()
write_styles()

def sidebar():
    cur_lang=display_lang_selector()

def ent_vis(text):
    from ipymarkup.palette import palette, BLUE, RED, GREEN
    from ipymarkup import box_markup
    spans = [(w['start'], w['end'], w['entity']) for w in extract_entities(text)]
    markup=box_markup(text, spans, palette=palette(PER=BLUE, ORG=RED, LOC=GREEN))
    html=''
    for line in markup.as_html:
        html=html+line
    # Newlines seem to mess with the rendering
    # html = html.replace("\n", " ")
    st.write(HTML_WRAPPER.format(html), unsafe_allow_html=True)

DEFAULT_TEXT = 'Rami Eid正在纽约石溪大学学习'
def main():
    sidebar()
    st.subheader("entity visualizer")
    text = st.text_area("Text to analyze", DEFAULT_TEXT)
    ent_vis(text)

if __name__ == '__main__':
    main()




from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.knowledge_base.actions import ActionQueryKnowledgeBase
from rasa_sdk.knowledge_base.storage import InMemoryKnowledgeBase
from pprint import pprint

class ActionHelloWorld(Action):

    def name(self) -> Text:
        return "action_hello_world"

    def run(self, dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        pprint(tracker.slots)
        dispatcher.utter_message("Hello World!")

        return []


class ActionMyKB(ActionQueryKnowledgeBase):
    def __init__(self):
        # load knowledge base with data from the given file
        # prefix='/pi/ws/sagas-ai/bots/saya'
        knowledge_base = InMemoryKnowledgeBase(f"seed/knowledge_base_data.json")

        # overwrite the representation function of the hotel object
        # by default the representation function is just the name of the object
        knowledge_base.set_representation_function_of_object(
            "hotel", lambda obj: obj["name"] + " (" + obj["city"] + ")"
        )

        super().__init__(knowledge_base)




from rasa.importers.utils import training_data_from_paths
import sagas.tracker_fn as tc

class NluDataProcs(object):
    def view_file(self, file, lang='en'):
        """
        $ python -m saai.nlu_data_procs view_file ./nlu_multilang/en/nlu_data.md
        $ python -m saai.nlu_data_procs view_file /pi/ws/knowledgebasebot/data/nlu.md

        :param file:
        :param lang:
        :return:
        """
        from pprint import pprint

        # files = ['./nlu_multilang/en/nlu_data.md']
        td = training_data_from_paths([file], language=lang)

        print('.. examples')
        # print(*[e.text for e in td.training_examples], sep='\n')
        print(*[(e.get("intent"), e.text) for e in td.training_examples], sep='\n')

        tc.emp('green', '.. intents')
        for intent in td.intents:
            tc.emp('yellow', f"  - {intent}")
        tc.emp('green', '.. entities')
        print(td.entities)

        tc.emp('green', '.. lookup_tables')
        pprint(td.lookup_tables)

    def train(self):
        from rasa.train import train_nlu
        prefix = '/pi/ws/sagas-ai/nlu_multilang'
        train_nlu(
            config=f"{prefix}/config_en.yml",
            nlu_data=f"{prefix}/en/",
            output=f'{prefix}/models',
            fixed_model_name='en_current',
            persist_nlu_training_data=True
        )



if __name__ == '__main__':
    import fire
    fire.Fire(NluDataProcs)




# coding=utf-8
from __future__ import unicode_literals

import csv
import re
from abc import ABCMeta, abstractmethod
from pathlib import Path

import six
from future.utils import with_metaclass

from snips_nlu.constants import (AUTOMATICALLY_EXTENSIBLE, DATA,
                                 SYNONYMS, USE_SYNONYMS,
                                 VALUE)
from snips_nlu.entity_parser.builtin_entity_parser import is_builtin_entity

PARSER_THRESHOLD = "parser_threshold"
AUTO_EXT_REGEX = re.compile(r'^#\sautomatically_extensible=(true|false)\s*$')


class Entity(with_metaclass(ABCMeta, object)):
    def __init__(self, name):
        self.name = name

    @abstractmethod
    def json(self):
        pass


class CustomEntity(Entity):
    """Custom entity of an :class:`.AssistantDataset`

        Attributes:
            utterances (list of :class:`.EntityUtterance`): entity utterances
            automatically_extensible (bool): whether or not the entity can be
                extended to values not present in the dataset
            use_synonyms (bool): whether or not to map entity values using
                synonyms
    """

    def __init__(self, name, utterances, automatically_extensible,
                 use_synonyms, parser_threshold=1.0):
        super(CustomEntity, self).__init__(name)
        self.utterances = utterances
        self.automatically_extensible = automatically_extensible
        self.use_synonyms = use_synonyms
        self.parser_threshold = parser_threshold

    @classmethod
    def from_file(cls, filepath):
        filepath = Path(filepath)
        stem = filepath.stem
        if not stem.startswith("entity_"):
            raise AssertionError("Entity filename should start with 'entity_' "
                                 "but found: %s" % stem)
        entity_name = stem[7:]
        if not entity_name:
            raise AssertionError("Entity name must not be empty")
        utterances = []
        with filepath.open(encoding="utf-8") as f:
            it = f
            if six.PY2:
                it = list(utf_8_encoder(it))
            reader = csv.reader(list(it))
            autoextent = True
            for row in reader:
                if six.PY2:
                    row = [cell.decode("utf-8") for cell in row]
                value = row[0]
                if reader.line_num == 1:
                    m = AUTO_EXT_REGEX.match(row[0])
                    if m:
                        autoextent = not m.group(1).lower() == 'false'
                        continue
                if len(row) > 1:
                    synonyms = row[1:]
                else:
                    synonyms = []
                utterances.append(EntityUtterance(value, synonyms))
        return cls(entity_name, utterances,
                   automatically_extensible=autoextent, use_synonyms=True)

    @property
    def json(self):
        """Returns the entity in json format"""
        return {
            AUTOMATICALLY_EXTENSIBLE: self.automatically_extensible,
            USE_SYNONYMS: self.use_synonyms,
            DATA: [u.json for u in self.utterances],
            PARSER_THRESHOLD: self.parser_threshold
        }


class EntityUtterance(object):
    """Represents a value of a :class:`.CustomEntity` with potential synonyms

    Attributes:
        value (str): entity value
        synonyms (list of str): The values to remap to the utterance value
        """

    def __init__(self, value, synonyms=None):
        self.value = value
        if synonyms is None:
            synonyms = []
        self.synonyms = synonyms

    @property
    def json(self):
        return {VALUE: self.value, SYNONYMS: self.synonyms}


class BuiltinEntity(Entity):
    """Builtin entity of an :class:`.AssistantDataset`"""

    @property
    def json(self):
        return dict()


def utf_8_encoder(f):
    for line in f:
        yield line.encode("utf-8")


def create_entity(entity_name, utterances=None, automatically_extensible=True,
                  use_synonyms=True):
    if is_builtin_entity(entity_name):
        return BuiltinEntity(entity_name)
    else:
        if utterances is None:
            utterances = []
        return CustomEntity(entity_name, utterances, automatically_extensible,
                            use_synonyms)



# ex: set ro:
# DO NOT EDIT.
# generated by smc (http://smc.sourceforge.net/)
# from file : AppClass.sm

import statemap


class AppClassState(statemap.State):

    def Entry(self, fsm):
        pass

    def Exit(self, fsm):
        pass

    def EOS(self, fsm):
        self.Default(fsm)

    def One(self, fsm):
        self.Default(fsm)

    def Unknown(self, fsm):
        self.Default(fsm)

    def Zero(self, fsm):
        self.Default(fsm)

    def Default(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write('TRANSITION   : Default\n')
        msg = "\n\tState: %s\n\tTransition: %s" % (
            fsm.getState().getName(), fsm.getTransition())
        raise statemap.TransitionUndefinedException(msg)

class Map1_Default(AppClassState):

    def Unknown(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Default\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Default.Unknown()\n")

        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("EXIT TRANSITION : Map1.Default.Unknown()\n")

        fsm.setState(Map1.Error)
        fsm.getState().Entry(fsm)


class Map1_Start(Map1_Default):

    def EOS(self, fsm):
        ctxt = fsm.getOwner()
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Start\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Start.EOS()\n")

        fsm.clearState()
        try:
            ctxt.Acceptable()
        finally:
            if fsm.getDebugFlag() == True:
                fsm.getDebugStream().write("EXIT TRANSITION : Map1.Start.EOS()\n")

            fsm.setState(Map1.OK)
            fsm.getState().Entry(fsm)


    def One(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Start\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Start.One()\n")

        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("EXIT TRANSITION : Map1.Start.One()\n")

        fsm.setState(Map1.Ones)
        fsm.getState().Entry(fsm)


    def Zero(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Start\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Start.Zero()\n")

        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("EXIT TRANSITION : Map1.Start.Zero()\n")

        fsm.setState(Map1.Zeros)
        fsm.getState().Entry(fsm)


class Map1_Zeros(Map1_Default):

    def EOS(self, fsm):
        ctxt = fsm.getOwner()
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Zeros\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Zeros.EOS()\n")

        fsm.clearState()
        try:
            ctxt.Acceptable()
        finally:
            if fsm.getDebugFlag() == True:
                fsm.getDebugStream().write("EXIT TRANSITION : Map1.Zeros.EOS()\n")

            fsm.setState(Map1.OK)
            fsm.getState().Entry(fsm)


    def One(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Zeros\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Zeros.One()\n")

        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("EXIT TRANSITION : Map1.Zeros.One()\n")

        fsm.setState(Map1.Ones)
        fsm.getState().Entry(fsm)


    def Zero(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Zeros\n")



class Map1_Ones(Map1_Default):

    def EOS(self, fsm):
        ctxt = fsm.getOwner()
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Ones\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Ones.EOS()\n")

        fsm.clearState()
        try:
            ctxt.Acceptable()
        finally:
            if fsm.getDebugFlag() == True:
                fsm.getDebugStream().write("EXIT TRANSITION : Map1.Ones.EOS()\n")

            fsm.setState(Map1.OK)
            fsm.getState().Entry(fsm)


    def One(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Ones\n")



    def Zero(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Ones\n")

        fsm.getState().Exit(fsm)
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Ones.Zero()\n")

        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("EXIT TRANSITION : Map1.Ones.Zero()\n")

        fsm.setState(Map1.Error)
        fsm.getState().Entry(fsm)


class Map1_OK(Map1_Default):
    pass

class Map1_Error(Map1_Default):

    def EOS(self, fsm):
        ctxt = fsm.getOwner()
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Error\n")

        endState = fsm.getState()
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("ENTER TRANSITION: Map1.Error.EOS()\n")

        fsm.clearState()
        try:
            ctxt.Unacceptable()
        finally:
            if fsm.getDebugFlag() == True:
                fsm.getDebugStream().write("EXIT TRANSITION : Map1.Error.EOS()\n")

            fsm.setState(endState)


    def One(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Error\n")



    def Unknown(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Error\n")



    def Zero(self, fsm):
        if fsm.getDebugFlag() == True:
            fsm.getDebugStream().write("LEAVING STATE   : Map1.Error\n")



class Map1(object):

    Start = Map1_Start('Map1.Start', 0)
    Zeros = Map1_Zeros('Map1.Zeros', 1)
    Ones = Map1_Ones('Map1.Ones', 2)
    OK = Map1_OK('Map1.OK', 3)
    Error = Map1_Error('Map1.Error', 4)
    Default = Map1_Default('Map1.Default', -1)

class AppClass_sm(statemap.FSMContext):

    def __init__(self, owner):
        statemap.FSMContext.__init__(self, Map1.Start)
        self._owner = owner

    def __getattr__(self, attrib):
        def trans_sm(*arglist):
            self._transition = attrib
            getattr(self.getState(), attrib)(self, *arglist)
            self._transition = None
        return trans_sm

    def enterStartState(self):
        self._state.Entry(self)

    def getOwner(self):
        return self._owner

# Local variables:
#  buffer-read-only: t
# End:



#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2000 - 2003 Charles W. Rapp.
# All Rights Reserved.
#
# Contributor(s):
#       Port to Python by Francois Perrad, francois.perrad@gadz.org
#
# Vehicle --
#
#  Draws a generic vehicle on the map (a black square) which
#  moves in straight lines along the road and obeys the stop light.
#
# RCS ID
# $Id$
#
# CHANGE LOG
# $Log$
# Revision 1.3  2014/07/12 08:21:44  fperrad
# compat python3
#
# Revision 1.2  2009/04/19 14:39:48  cwrapp
# Added call to enterStartState before issuing first FSM transition.
#
# Revision 1.1  2005/05/28 17:48:30  cwrapp
# Added Python examples 1 - 4 and 7.
#
#

# from tkinter import *

from fsm.ex4 import Vehicle_sm

class Vehicle:

	_speed = 2

	def __init__(self, stoplight, direction, canvas):
		self._fsm = Vehicle_sm.Vehicle_sm(self)

		# The canvas to draw on and the direction this vehicle is
		# moving.
		self._canvas = canvas
		self._direction = direction

		# The stoplight object is responsible knowing the road
		# layout. Ask it for all relevant information.
		self._stoplight = stoplight

		# This vehicle is initially at the road's outside edge.
		# Figure out the road's length.
		XLength = stoplight.getRoadLengthX()
		YLength = stoplight.getRoadLengthY()
		LaneWidth = stoplight.getRoadWidth() / 2

		# The vehicle is 12 pixels x 12 pixels.
		self._vehicleSize = 6

		# A 3 pixel separation is to be maintained between vehicles.
		self._vehicleSeparation = 3

		# How far away the vehicle is from the curb.
		CurbOffset = (LaneWidth - self._vehicleSize) / 2

		# The vehicle's current canvas location. This is the
		# square's upper left hand corner.
		if      direction == 'north':
			self._xpos = (XLength / 2) + CurbOffset
			self._ypos = YLength - self._vehicleSize
		elif direction == 'south':
			self._xpos = (XLength / 2) - LaneWidth + CurbOffset
			self._ypos = 0
		elif direction == 'east':
			self._xpos = 0
			self._ypos = (YLength / 2) + CurbOffset
		elif direction == 'west':
			self._xpos = XLength - self._vehicleSize
			self._ypos = (YLength / 2) - LaneWidth + CurbOffset

		# Put the vehicle on display.
		self._canvasID = canvas.create_rectangle(
				self._xpos,
				self._ypos,
				self._xpos + self._vehicleSize,
				self._ypos + self._vehicleSize,
				fill='black',
				outline='white',
		)

		# Move this vehicle along at near movie-refresh rate.
		self._redrawRate = 1000 / 60

		# Store the after's timer ID here.
		self._timerID = -1

		# Set this flag to true when the vehicle has
		# completed its trip.
		self._isDoneFlag = False

		# Uncomment to see debug output.
		#self._fsm.setDebugFlag(True)

	def Delete(self):
		if self._timerID >= 0:
			self._canvas.after_cancel(self._timerID)
			self._timerID = -1
		self._canvas.delete(self._canvasID)

	# timeout --
	#
	#   If the vehicle has driven off the canvas, then
	#   delete the vehicle.
	#   Check if the vehicle is at the intersection and the
	#   light is either yellow or red. If yes, then issue a
	#   "LightRed" transition. If all is go, then keep on
	#   truckin.
	#
	# Arugments:
	#   None.

	def timeout(self):
		self._timerID = -1
		if self.OffCanvas():
			self._fsm.TripDone()
		elif self.AtIntersection() and self.getLight() != 'green':
			self._fsm.LightRed()
		else:
			self._fsm.KeepGoing()

	def getLight(self):
		return self._stoplight.getLight(self._direction)

	# lightGreen --
	#
	#   The light has turned green. Time to get moving again.
	#
	# Arguments:
	#   None

	def lightGreen(self):
		self._fsm.LightGreen()

	# setSpeed --
	#
	#   Set speed for all vehicles.
	#
	# Arguments:
	#   speed   In pixels.

	def setSpeed(klass, speed):
		if speed < 1 or speed > 10:
			print("Invalid speed (%d).\n" % speed)
		else:
			klass._speed = speed

	setSpeed = classmethod(setSpeed)

	# isDone --
	#
	#   Has this vehicle completed its trip?
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Returns true if the trip is done and false
	#   otherwise.

	def isDone(self):
		return self._isDoneFlag

	# start --
	#
	#   Start this vehicle running.
	#
	# Arguments:
	#   None.

	def Start(self):
		self._fsm.enterStartState()
		self._fsm.Start()

	# pause --
	#
	#   Pause this vehicles' running.
	#
	# Arguments:
	#   None.

	def Pause(self):
		self._fsm.Pause()

	# continue --
	#
	#   Continue this vehicles' running.
	#
	# Arguments:
	#   None.

	def Continue(self):
		self._fsm.Continue()

	# stop --
	#
	#   Stop this vehicles' running.
	#
	# Arguments:
	#   None.
	#

	def Stop(self):
		self._fsm.Stop()
		self.Delete()

	# State Machine Actions
	#
	# The following methods are called by the state machine.

	# SetTimer --
	#
	#   Set the timer for the next move.
	#
	# Arguments:
	#   None.

	def SetTimer(self):
		self._timerID = self._canvas.after(self._redrawRate, self.timeout)

	# StopTimer --
	#
	#   Stop the vehicle's timer.
	#
	# Arguments:
	#   None.

	def StopTimer(self):
		if self._timerID >= 0:
			self._canvas.after_cancel(self._timerID)
			self._timerID = -1

	# Move --
	#
	#   1. Calculate the vehicle's new position.
	#   2. Remove the vehicle from the canvas.
	#   3. Draw the vehicles new position.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   None returned. Side affect of redrawing vehicle.

	def Move(self):
		if      self._direction == 'north':
			Xmove = 0
			Ymove = - self._speed
		elif self._direction == 'south':
			Xmove = 0
			Ymove = self._speed
		elif self._direction == 'east':
			Xmove = self._speed
			Ymove = 0
		elif self._direction == 'west':
			Xmove = - self._speed
			Ymove = 0

		self._canvas.move(self._canvasID, Xmove, Ymove)

		self._xpos += Xmove
		self._ypos += Ymove

	# RegisterWithLight --
	#
	#   When the light turns green, it will inform us.
	#
	# Arguments:
	#   None.

	def RegisterWithLight(self):
		self._stoplight.registerVehicle(self, self._direction)

	# SelfDestruct --
	#
	#   Remove the vehicle from the canvas.
	#
	# Arguments:
	#   None.

	def SelfDestruct(self):
		self._canvas.delete(self._canvasID)
		self._canvasID = -1
		self._isDoneFlag = True

	# OffCanvas --
	#
	#   Figure out if the vehicle has driven off the map.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Returns true if the vehicle is off the map; otherwise
	#   false.

	def OffCanvas(self):
		if      self._direction == 'north':
			return (self._ypos - self._speed) <= 0
		elif self._direction == 'south':
			YLength = self._stoplight.getRoadLengthY()
			return (self._ypos + self._speed) >= YLength
		elif self._direction == 'east':
			XLength = self._stoplight.getRoadLengthX()
			return (self._xpos + self._speed) >= XLength
		elif self._direction == 'west':
			return (self._xpos - self._speed) <= 0

	# AtIntersection --
	#
	#   Figure out whether this vehicile is at the intersection
	#   or not.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Returns true if the vehicle is at the intersection;
	#   otherwise, false.

	def AtIntersection(self):
		# The vehicle is not at the intersection until proven
		# otherwise.
		Retval = False

		XLength = self._stoplight.getRoadLengthX()
		YLength = self._stoplight.getRoadLengthY()
		LaneWidth = self._stoplight.getRoadWidth() / 2

		# Calculate the intersections coordinates based on
		# the vehicle's direction. Then calculate where the
		# vehicle will end up this move. If the vehicle will
		# move beyond the intersection stop line, then the
		# vehicle is at the intersection.
		#
		# Also take into account the vehicles already waiting
		# at the intersection.
		#
		# By the way, once the vehicle moves past the intersection,
		# ignore the light.
		NumVehicles = self._stoplight.getQueueSize(self._direction)
		LenVehicles = (self._vehicleSize + self._vehicleSeparation) * NumVehicles
		if      self._direction == 'north':
			YIntersection = (YLength / 2) + LaneWidth + (self._vehicleSize / 2) + LenVehicles
			Retval = (self._ypos > YIntersection) and (self._ypos - self._speed <= YIntersection)
		elif self._direction == 'south':
			YIntersection = (YLength / 2) - LaneWidth - (self._vehicleSize / 2) - LenVehicles
			Retval = (self._ypos < YIntersection) and (self._ypos + self._speed >= YIntersection)
		elif self._direction == 'east':
			XIntersection = (XLength / 2) - LaneWidth - (self._vehicleSize / 2) - LenVehicles
			Retval = (self._xpos < XIntersection) and (self._xpos + self._speed >= XIntersection)
		elif self._direction == 'west':
			XIntersection = (XLength / 2) + LaneWidth + (self._vehicleSize / 2) + LenVehicles
			Retval = (self._xpos > XIntersection) and (self._xpos - self._speed <= XIntersection)
		return Retval



from typing import List, Dict, Any, Optional, Text

import asyncio
import glob
from rasa.train import train_async
import rasa.utils.io as io_utils
import logging

from rasa.utils.endpoints import ClientResponseError, EndpointConfig
from rasa.core.agent import Agent
from rasa.core.interpreter import RasaNLUInterpreter
from rasa.model import get_model, get_latest_model
from rasa.core.channels.channel import UserMessage

logger = logging.getLogger(__name__)

class BotsConf(object):
    def __init__(self, conf='agents.json'):
        # import json_utils
        from python_json_config import ConfigBuilder
        from saai.saai_conf import saai_conf

        # self.endpoint = EndpointConfig("http://localhost:5055/webhook")
        self.conf=f"{saai_conf.runtime_dir}/conf/{conf}"
        # conf_data=json_utils.read_json_file(self.conf)
        builder = ConfigBuilder()
        self.config = builder.parse_config(self.conf)
        # bot_locs={'genesis': '/pi/ws/sagas-ai/bots/genesis'}
        # self.bot_locs = conf_data['bot_locs']
        # self.config_file=conf_data['config_file']

        self.templates_dir=f'{saai_conf.runtime_dir}/templates'
        self.ruleset_files='/pi/stack/conf/ruleset_*.json'

    def get_loc(self, bot:Text):
        return self.config.get(bot).location

    def get_config(self, bot:Text):
        return self.config.get(bot).config_file

    def get_endpoint(self, bot:Text):
        from sagas.conf.runtime import runtime
        bot_endpoint=f"{bot}_actions" if runtime.is_docker() else 'localhost'
        return EndpointConfig(f"http://{bot_endpoint}:5055/webhook")

def generate_domain_file(conf:BotsConf):
    cnt = io_utils.read_yaml_file(f'{conf.templates_dir}/domain.yml')
    intents = cnt['intents']
    actions = cnt['actions']
    for f in glob.glob(conf.ruleset_files):
        rules = io_utils.read_json_file(f)
        for rule in rules:
            intents.append({rule['intent']: {'triggers': rule['action']}})
            actions.append(rule['action'])
    return cnt

async def train_agent(bot:Text, conf:BotsConf):
    cnt=generate_domain_file(conf)

    prefix = f"{conf.get_loc(bot)}"
    # domain_file="./out/domain_1.yml"
    domain_file = f"{prefix}/domain.yml"
    io_utils.write_yaml_file(cnt, domain_file)
    await train_async(
        domain=domain_file,
        config=f"{prefix}/{conf.get_config(bot)}",
        training_files=f"{prefix}/data/",
        output_path=f"{prefix}/models/",
    )

async def load_agent(bot:Text, conf:BotsConf) -> Agent:
    # train it
    await train_agent(bot, conf)
    # load it
    bot_loc = get_latest_model(f"{conf.get_loc(bot)}/models")
    print(f'.. load bot model {bot_loc}')
    agent = Agent.load(bot_loc, action_endpoint=conf.get_endpoint(bot))
    return agent

agents={}
async def get_agent(bot:Text, conf:BotsConf, force=False) -> Agent:
    if force or (bot not in agents):
        agents[bot]=await load_agent(bot, conf)
    return agents[bot]

async def handle_message(mod, text, sender, conf:BotsConf):
    agent=await get_agent(mod, conf)
    message = UserMessage(text, sender_id=sender)
    return await agent.handle_message(message)

class AgentProcs(object):
    def env(self):
        """
        $ python -m saai.agent_procs env
        :return:
        """
        conf=BotsConf()
        print(conf.get_endpoint('genesis').url)
        print(conf.get_loc('genesis'))
        print(conf.get_config('genesis'))

    def tests(self, bot='genesis'):
        """
        $ python -m saai.agent_procs tests
        :param bot:
        :return:
        """
        text = '/behave_purpose{"object_type": "restaurant"}'
        loop = asyncio.get_event_loop()
        return loop.run_until_complete(handle_message(bot, text, sender='default', conf=BotsConf()))

if __name__ == '__main__':
    import fire
    from sagas.tool.loggers import init_logger

    init_logger()
    fire.Fire(AgentProcs)





#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2005. Charles W. Rapp.
# All Rights Reserved.
#
# Port to Python by Francois Perrad, francois.perrad@gadz.org
# Copyright 2004, Francois Perrad.
# All Rights Reserved.
#
# Contributor(s):
#
# RCS ID
# Id: statemap.py,v 1.8 2010/09/11 18:56:57 fperrad Exp
#
# See: http://smc.sourceforge.net/
#

import sys

class StateUndefinedException(Exception):
	"""A StateUndefinedException is thrown by
	an SMC-generated state machine whenever a transition is taken
	and there is no state currently set. This occurs when a
	transition is issued from within a transition action."""
	pass

class TransitionUndefinedException(Exception):
	"""A TransitionUndefinedException is thrown by
	an SMC-generated state machine whenever a transition is taken
	which:

	 - Is not explicitly defined in the current state.
	 - Is not explicitly defined in the current FSM's default state.
	 - There is no Default transition in the current state."""
	pass


class State(object):
	"""base State class"""

	def __init__(self, name, id):
		self._name = name
		self._id = id

	def getName(self):
		"""Returns the state's printable name."""
		return self._name

	def getId(self):
		"""Returns the state's unique identifier."""
		return self._id


class FSMContext(object):
	"""The user can derive FSM contexts from this class and interface
	to them with the methods of this class.

	The finite state machine needs to be initialized to the starting
	state of the FSM.  This must be done manually in the constructor
	of the derived class.
	"""

	def __init__(self, state):
		self._state = state
		self._previous_state = None
		self._state_stack = []
		self._transition = None
		self._debug_flag = False
		self._debug_stream = sys.stderr

	def getDebugFlag(self):
		"""Returns the debug flag's current setting."""
		return self._debug_flag

	def setDebugFlag(self, flag):
		"""Sets the debug flag.
		A true value means debugging is on and false means off."""
		self._debug_flag = flag

	def getDebugStream(self):
		"""Returns the stream to which debug output is written."""
		return self._debug_stream

	def setDebugStream(self, stream):
		"""Sets the debug output stream."""
		self._debug_stream = stream

	def getState(self):
		"""Returns the current state."""
		if self._state == None:
			raise StateUndefinedException
		return self._state

	def isInTransition(self):
		"""Is this state machine already inside a transition?
		True if state is undefined."""
		if self._state == None:
			return True
		else:
			return False

	def getTransition(self):
		"""Returns the current transition's name.
		Used only for debugging purposes."""
		return self._transition

	def clearState(self):
		"""Clears the current state."""
		self._previous_state = self._state
		self._state = None


	def getPreviousState(self):
		"""Returns the state which a transition left.
		May be None"""
		return self._previous_state


	def setState(self, state):
		"""Sets the current state to the specified state."""
		if not isinstance(state, State):
			raise ValueError("state should be a statemap.State")
		self._state = state
		if self._debug_flag:
			self._debug_stream.write("ENTER STATE     : %s\n" % self._state.getName())

	def isStateStackEmpty(self):
		"""Returns True if the state stack is empty and False otherwise."""
		return len(self._state_stack) == 0

	def getStateStackDepth(self):
		"""Returns the state stack's depth."""
		return len(self._state_stack)

	def pushState(self, state):
		"""Push the current state on top of the state stack
		and make the specified state the current state."""
		if not isinstance(state, State):
			raise ValueError("state should be a statemap.State")
		if self._state != None:
			self._state_stack.append(self._state)
		self._state = state
		if self._debug_flag:
			self._debug_stream.write("PUSH TO STATE   : %s\n" % self._state.getName())

	def popState(self):
		"""Make the state on top of the state stack the current state."""
		if len(self._state_stack) == 0:
			if self._debug_flag:
				self._debug_stream.write("POPPING ON EMPTY STATE STACK.\n")
			raise ValueError("empty state stack")
		else:
			self._state = self._state_stack.pop()
			if self._debug_flag:
				self._debug_stream.write("POP TO STATE    : %s\n" % self._state.getName())

	def emptyStateStack(self):
		"""Remove all states from the state stack."""
		self._state_stack = []



#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is State Machine Compiler (SMC).
#
# The Initial Developer of the Original Code is Charles W. Rapp.
# Portions created by Charles W. Rapp are
# Copyright (C) 2000 - 2003 Charles W. Rapp.
# All Rights Reserved.
#
# Contributor(s):
#       Port to Python by Francois Perrad, francois.perrad@gadz.org
#
# Stoplight --
#
#  When a timer goes off, change the light's color as per the
#  state machine.
#
# RCS ID
# $Id$
#
# CHANGE LOG
# $Log$
# Revision 1.2  2009/04/19 14:39:48  cwrapp
# Added call to enterStartState before issuing first FSM transition.
#
# Revision 1.1  2005/05/28 17:48:29  cwrapp
# Added Python examples 1 - 4 and 7.
#
#

from fsm.ex4 import Stoplight_sm

class Stoplight:

	def __init__(self, canvas):
		self._canvas = canvas
		# Create the stop light's state machine.
		self._fsm = Stoplight_sm.Stoplight_sm(self)

		self._east_light = dict()
		self._west_light = dict()
		self._north_light = dict()
		self._south_light = dict()
		self._roadWidth = 38
		self._lightDiameter = 6
		self._lightSpace = 2

		# Set the light height and width.
		self._lightWidth = self._lightDiameter + self._lightSpace * 2
		self._lightHeight = self._lightDiameter * 3 + self._lightSpace * 4

		self._northVehicleList = []
		self._southVehicleList = []
		self._eastVehicleList = []
		self._westVehicleList = []

		# Create the stoplight GUI. Draw the roads.
		self.DrawRoads()

		# Draw the stoplights.
		self.DrawLights()

		# Set each light timer.
		self._timeouts = {
				'NSGreenTimer': 7000,
				'EWGreenTimer': 5000,
				'YellowTimer': 2000,
		}

		self._timerID = -1

		# Uncomment to see debug output.
		#self._fsm.setDebugFlag(True)

	# getRoadLengthX --
	#
	#   Return the road's length in X direction.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Pixel length of road in X direction.

	def getRoadLengthX(self):
		return int(self._canvas.cget('width'))

	# getRoadLengthY --
	#
	#   Return the road's length in Y direction in pixels.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Pixel length of road in Y direction.

	def getRoadLengthY(self):
		return int(self._canvas.cget('height'))

	# getRoadWidth --
	#
	#   Return road's width in pixels.
	#
	# Arguments:
	#   None.
	#
	# Results:
	#   Road's width in pixels.

	def getRoadWidth(self):
		return self._roadWidth

	# getLight --
	#
	#   Return a specified stop lights color.
	#
	# Arguments:
	#   direction   Must be either north, south east or west.
	#
	# Results:
	#   Returns the color for that direction.

	def getLight(self, direction):
		cv = self._canvas

		# The direction represents which way the vehicle
		# is facing. This is the opposite direction in which
		# the light is facing.
		if      direction == 'north':
			RedLight = cv.itemcget(self._south_light['RED'], 'fill')
			YellowLight = cv.itemcget(self._south_light['YELLOW'], 'fill')
			GreenLight = cv.itemcget(self._south_light['GREEN'], 'fill')
		elif direction == 'south':
			RedLight = cv.itemcget(self._north_light['RED'], 'fill')
			YellowLight = cv.itemcget(self._north_light['YELLOW'], 'fill')
			GreenLight = cv.itemcget(self._north_light['GREEN'], 'fill')
		elif direction == 'east':
			RedLight = cv.itemcget(self._west_light['RED'], 'fill')
			YellowLight = cv.itemcget(self._west_light['YELLOW'], 'fill')
			GreenLight = cv.itemcget(self._west_light['GREEN'], 'fill')
		elif direction == 'west':
			RedLight = cv.itemcget(self._east_light['RED'], 'fill')
			YellowLight = cv.itemcget(self._east_light['YELLOW'], 'fill')
			GreenLight = cv.itemcget(self._east_light['GREEN'], 'fill')

		if RedLight == 'red':
			return 'red'
		elif YellowLight == 'yellow':
			return 'yellow'
		else:
			return 'green'

	# registerVehicle --
	#
	#   A vehicle is waiting for this light to turn green.
	#   Add it to the list. When the light turns green,
	#   the vehicle will be told about it.
	#
	# Arguments:
	#   vehicle    A vehicle object name.
	#   direction  The direction the vehicle is moving.

	def registerVehicle(self, vehicle, direction):
		if      direction == 'north':
			self._northVehicleList.append(vehicle)
		elif direction == 'south':
			self._southVehicleList.append(vehicle)
		elif direction == 'east':
			self._eastVehicleList.append(vehicle)
		elif direction == 'west':
			self._westVehicleList.append(vehicle)

	# getQueueSize --
	#
	#   Return the number of vehicles waiting on a red in
	#   a particular direction.
	#
	# Arguments:
	#   direction   The direction the vehicle is moving.
	#
	# Results:
	#   The size of the red light queue for that direction.

	def getQueueSize(self, direction):
		if      direction == 'north':
			return len(self._northVehicleList)
		elif direction == 'south':
			return len(self._southVehicleList)
		elif direction == 'east':
			return len(self._eastVehicleList)
		elif direction == 'west':
			return len(self._westVehicleList)

	# setLightTimer --
	#
	#   Set a particular light's timer. The value is given in
	#   seconds, so convert to milliseconds.
	#
	# Arguments:
	#   light    NSGreenTimer, EWGreenTimer or YellowTimer.
	#   time     Light time in seconds.

	def setLightTimer(self, light, time):
		self._timeouts[light] = time * 1000

	# start --
	#
	#   Start the demo running.
	#
	# Arguments:
	#   None.

	def Start(self):
		self._fsm.enterStartState()
		self._fsm.Start()

	# pause --
	#
	#   Pause this demo.
	#
	# Arguments:
	#   None.

	def Pause(self):
		self._fsm.Pause()

	# continue --
	#
	#   Continue this demo.
	#
	# Arguments:
	#   None.

	def Continue(self):
		self._fsm.Continue()

	# stop --
	#
	#   Stop this demo.
	#
	# Arguments:
	#   None.

	def Stop(self):
		self._fsm.Stop()

	# State Machine Actions.
	#
	# The following methods are called by the state machine..

	def TurnLight(self, direction, color):
		cv = self._canvas
		if      direction == 'EWLIGHT':
			if      color == 'red':
				cv.itemconfigure(self._east_light['YELLOW'], fill='white')
				cv.itemconfigure(self._west_light['YELLOW'], fill='white')
				cv.itemconfigure(self._east_light['RED'], fill='red')
				cv.itemconfigure(self._west_light['RED'], fill='red')
			elif color == 'green':
				cv.itemconfigure(self._east_light['RED'], fill='white')
				cv.itemconfigure(self._west_light['RED'], fill='white')
				cv.itemconfigure(self._east_light['GREEN'], fill='green')
				cv.itemconfigure(self._west_light['GREEN'], fill='green')
			elif color  == 'yellow':
				cv.itemconfigure(self._east_light['GREEN'], fill='white')
				cv.itemconfigure(self._west_light['GREEN'], fill='white')
				cv.itemconfigure(self._east_light['YELLOW'], fill='yellow')
				cv.itemconfigure(self._west_light['YELLOW'], fill='yellow')
		elif direction == 'NSLIGHT':
			if      color == 'red':
				cv.itemconfigure(self._north_light['YELLOW'], fill='white')
				cv.itemconfigure(self._south_light['YELLOW'], fill='white')
				cv.itemconfigure(self._north_light['RED'], fill='red')
				cv.itemconfigure(self._south_light['RED'], fill='red')
			elif color == 'green':
				cv.itemconfigure(self._north_light['RED'], fill='white')
				cv.itemconfigure(self._south_light['RED'], fill='white')
				cv.itemconfigure(self._north_light['GREEN'], fill='green')
				cv.itemconfigure(self._south_light['GREEN'], fill='green')
			elif color == 'yellow':
				cv.itemconfigure(self._north_light['GREEN'], fill='white')
				cv.itemconfigure(self._south_light['GREEN'], fill='white')
				cv.itemconfigure(self._north_light['YELLOW'], fill='yellow')
				cv.itemconfigure(self._south_light['YELLOW'], fill='yellow')

	def SetTimer(self, timer):
		self._timerID = self._canvas.after(self._timeouts[timer], self.Timeout)

	def StopTimer(self):
		if self._timerID >= 0:
			self._canvas.after_cancel(self._timerID)
			self._timerID = -1

	def Timeout(self):
		self._timerID = -1
		self._fsm.Timeout()

	def ResetLights(self):
		cv = self._canvas

		cv.itemconfigure(self._east_light['YELLOW'], fill='white')
		cv.itemconfigure(self._west_light['YELLOW'], fill='white')
		cv.itemconfigure(self._east_light['RED'], fill='white')
		cv.itemconfigure(self._west_light['RED'], fill='white')
		cv.itemconfigure(self._east_light['GREEN'], fill='white')
		cv.itemconfigure(self._west_light['GREEN'], fill='white')

		cv.itemconfigure(self._north_light['YELLOW'], fill='white')
		cv.itemconfigure(self._south_light['YELLOW'], fill='white')
		cv.itemconfigure(self._north_light['RED'], fill='white')
		cv.itemconfigure(self._south_light['RED'], fill='white')
		cv.itemconfigure(self._north_light['GREEN'], fill='white')
		cv.itemconfigure(self._south_light['GREEN'], fill='white')

	# InformVehicles --
	#
	#   Tell the vehicles that were waiting on the green light
	#   that they can go now.
	#
	# Arguments:
	#   direction   Which light turned green.

	def InformVehicles(self, direction):
		if      direction == 'north':
			for vehicle in self._northVehicleList:
				vehicle.lightGreen()
			self._northVehicleList = []
		elif direction == 'south':
			for vehicle in self._southVehicleList:
				vehicle.lightGreen()
			self._southVehicleList = []
		elif direction == 'east':
			for vehicle in self._eastVehicleList:
				vehicle.lightGreen()
			self._eastVehicleList = []
		elif direction == 'west':
			for vehicle in self._westVehicleList:
				vehicle.lightGreen()
			self._westVehicleList = []

	def DrawRoads(self):
		cv = self._canvas
		# The roads are drawn as follows:
		#
		#        (x2,y1)   (x4,y1)
		#             |  |  |
		#             |     |
		#             |  |  |
		# (x1,y2)     |     |       (x5,y2)
		# ------------+  |  +------------
		#         (x2,y2) (x4,y2)
		# - - - - - -        - - - - - -
		#         (x2,y4) (x4,y4)   (x5,y4)
		# ------------+     +------------
		# (x1,y4)     |  |  |
		#             |     |
		#             |  |  |
		#             |     |
		#        (x2,y5) |(x4,y5)

		# Calculate the line segment's length.
		XLength = (self.getRoadLengthX() / 2) - self._roadWidth / 2
		YLength = (self.getRoadLengthY() / 2) - self._roadWidth / 2

		# Calculate the major coordinates.
		X1 = 0
		Y1 = 0
		X2 = XLength
		Y2 = YLength
		X3 = int(cv.cget('width')) / 2
		Y3 = int(cv.cget('height')) / 2
		X4 = int(cv.cget('width')) - XLength
		Y4 = int(cv.cget('height')) - YLength
		X5 = int(cv.cget('width'))
		Y5 = int(cv.cget('height'))

		# Put green lawns around the road.
		cv.create_rectangle(X1, Y1, X2, Y2,
			outline="",
			fill='green',
		)
		cv.create_rectangle(X1, Y4, X2, Y5,
			outline="",
			fill='green',
		)
		cv.create_rectangle(X4, Y4, X5, Y5,
			outline="",
			fill='green',
		)
		cv.create_rectangle(X4, Y1, X5, Y2,
			outline="",
			fill='green',
		)

		# Draw four connected lines where each drawing uses three
		# coordinates.
		cv.create_line(X1, Y2, X2, Y2, X2, Y1)
		cv.create_line(X4, Y1, X4, Y2, X5, Y2)
		cv.create_line(X1, Y4, X2, Y4, X2, Y5)
		cv.create_line(X4, Y5, X4, Y4, X5, Y4)

		# Now draw the lane markings.
		cv.create_line(X1, Y3, X2, Y3)
		cv.create_line(X3, Y1, X3, Y2)
		cv.create_line(X4, Y3, X5, Y3)
		cv.create_line(X3, Y4, X3, Y5)

	def DrawLights(self):
		cv = self._canvas
		# The lights are drawns as follows:
		#
		#  y1          +---+
		#              | o |green
		#              | o |yellow
		#              | o |red
		#  y2  +-------+---+-------+
		#      | o o o |   | o o o |
		#  y3  +-------+---+-------+
		#              | o |red
		#              | o |yellow
		#              | o |green
		#  y4          +---+
		#
		#    x1       x2   x3     x4
		# Store each light as a separate element in a table.

		# Figure out the coordinates for the stoplights.
		X1 = int(cv.cget('width')) / 2 - self._lightWidth / 2 - self._lightHeight
		Y1 = int(cv.cget('height')) / 2 - self._lightWidth / 2 - self._lightHeight
		X2 = X1 + self._lightHeight
		Y2 = Y1 + self._lightHeight
		X3 = X2 + self._lightWidth
		Y3 = Y2 + self._lightWidth
		X4 = X3 + self._lightHeight
		Y4 = Y3 + self._lightHeight

		# Draw the four stop lights boxes.
		cv.create_rectangle(X2, Y1, X3, Y2,
				outline='black',
				fill='black',
				width=1,
		)
		cv.create_rectangle(X1, Y2, X2, Y3,
				outline='black',
				fill='black',
				width=1,
		)
		cv.create_rectangle(X2, Y3, X3, Y4,
				outline='black',
				fill='black',
				width=1,
		)
		cv.create_rectangle(X3, Y2, X4, Y3,
				outline='black',
				fill='black',
				width=1,
		)

		# Draw the lights within the stoplights. Save the
		# canvas items into an array because they will be
		# referenced later. Because there are two lights
		self._north_light['RED'] = cv.create_oval(
				X2 + self._lightSpace,
				Y1 + self._lightSpace,
				X3 - self._lightSpace,
				Y1 + self._lightSpace + self._lightDiameter,
				outline='black',
				fill='white'
		)
		self._north_light['YELLOW'] = cv.create_oval(
				X2 + self._lightSpace,
				Y1 + self._lightSpace * 2 + self._lightDiameter,
				X3 - self._lightSpace,
				Y1 + self._lightSpace * 2 + self._lightDiameter * 2,
				outline='black',
				fill='white'
		)
		self._north_light['GREEN'] = cv.create_oval(
				X2 + self._lightSpace,
				Y1 + self._lightSpace * 3 + self._lightDiameter * 2,
				X3 - self._lightSpace,
				Y1 + self._lightSpace * 3 + self._lightDiameter * 3,
				outline='black',
				fill='white'
		)

		self._west_light['RED'] = cv.create_oval(
				X1 + self._lightSpace,
				Y2 + self._lightSpace,
				X1 + self._lightSpace + self._lightDiameter,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)
		self._west_light['YELLOW'] = cv.create_oval(
				X1 + self._lightSpace * 2 + self._lightDiameter,
				Y2 + self._lightSpace,
				X1 + self._lightSpace * 2 + self._lightDiameter * 2,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)
		self._west_light['GREEN'] = cv.create_oval(
				X1 + self._lightSpace * 3 + self._lightDiameter * 2,
				Y2 + self._lightSpace,
				X1 + self._lightSpace * 3 + self._lightDiameter * 3,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)

		self._south_light['GREEN'] = cv.create_oval(
				X2 + self._lightSpace,
				Y3 + self._lightSpace,
				X3 - self._lightSpace,
				Y3 + self._lightSpace + self._lightDiameter,
				outline='black',
				fill='white'
		)
		self._south_light['YELLOW'] = cv.create_oval(
				X2 + self._lightSpace,
				Y3 + self._lightSpace * 2 + self._lightDiameter,
				X3 - self._lightSpace,
				Y3 + self._lightSpace * 2 + self._lightDiameter * 2,
				outline='black',
				fill='white'
		)
		self._south_light['RED'] = cv.create_oval(
				X2 + self._lightSpace,
				Y3 + self._lightSpace * 3 + self._lightDiameter * 2,
				X3 - self._lightSpace,
				Y3 + self._lightSpace * 3 + self._lightDiameter * 3,
				outline='black',
				fill='white'
		)

		self._east_light['GREEN'] = cv.create_oval(
				X3 + self._lightSpace,
				Y2 + self._lightSpace,
				X3 + self._lightSpace + self._lightDiameter,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)
		self._east_light['YELLOW'] = cv.create_oval(
				X3 + self._lightSpace * 2 + self._lightDiameter,
				Y2 + self._lightSpace,
				X3 + self._lightSpace * 2 + self._lightDiameter * 2,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)
		self._east_light['RED'] = cv.create_oval(
				X3 + self._lightSpace * 3 + self._lightDiameter * 2,
				Y2 + self._lightSpace,
				X3 + self._lightSpace * 3 + self._lightDiameter * 3,
				Y3 - self._lightSpace,
				outline='black',
				fill='white'
		)



from typing import Any, Dict, List, Text

from rasa.nlu.extractors import EntityExtractor
from rasa.nlu.training_data import Message

def extract_entities(sents):
    import jieba.posseg as pseg
    words = pseg.cut(sents,use_paddle=True)
    tokens=[(word, flag) for word, flag in words]
    running_offset = 0
    rs = []
    for token in tokens:
        word = token[0]
        word_offset = sents.index(word, running_offset)
        word_len = len(word)
        running_offset = word_offset + word_len
        rs.append({"start": word_offset,
                   "end": running_offset,
                   'value': word,
                   'entity': token[1],
                   "confidence": None,
                   })
    return [w for w in rs if w['entity'] in {'PER', 'LOC', 'ORG', 'TIME'}]

class JiebaEntityExtractor(EntityExtractor):

    provides = ["entities"]
    defaults = {
        # by default all dimensions recognized
        # dimensions can be configured to contain an array of strings
        # with the names of the dimensions to filter for
        "dimensions": None
    }

    def __init__(self, component_config: Text = None) -> None:
        super().__init__(component_config)

    def process(self, message: Message, **kwargs: Any) -> None:
        all_extracted = self.add_extractor_name(extract_entities(message.text))
        dimensions = self.component_config["dimensions"]
        extracted = JiebaEntityExtractor.filter_irrelevant_entities(
            all_extracted, dimensions
        )
        message.set(
            "entities", message.get("entities", []) + extracted,
            add_to_output=True
        )





import os
import setuptools
from setuptools import find_packages, setup, Command

here = os.path.abspath(os.path.dirname(__file__))

# Avoids IDE errors, but actual version is read from version.py
__version__ = None
with open("saai/version.py") as f:
    exec(f.read())

with open("README.md", "r") as fh:
    long_description = fh.read()

# What packages are required for this module to be executed?
REQUIRED = [
    "jieba",
    "simplejson",
    "paddlepaddle-tiny",
    "sagas>=0.1.2",
    "python_json_config",
    "python-dotenv",
]

setuptools.setup(
    name="saai",
    version=__version__,
    author="Samlet Wu",
    author_email="xiaofei.wu@gmail.com",
    description="nlu components",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/samlet/saai",
    packages=["", *find_packages(exclude=('bots', 'tests', 'fsm', ))],
    entry_points={"console_scripts": ["saai=saai.__main__:main",
                                      "saash=saai.shell.__main__:shell_main",
                                      ]},
    install_requires=REQUIRED,
    extras_require={
        'rasa': ['rasa~=1.6.0', 'rasa-sdk~=1.6.0'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    package_data={
              'saai': ['conf/*', 'templates/*'],
              'data': ['knowledge-base/*']},
    # $ setup.py publish support.
    # cmdclass={
    #     'upload': UploadCommand,
    # },
    project_urls={
        "Bug Reports": "https://github.com/samlet/saai/issues",
        "Source": "https://github.com/samlet/saai",
    },
)




class SaaiConf(object):
    @property
    def runtime_dir(self):
        """
        >>> from saai.saai_conf import saai_conf
        >>> saai_conf.runtime_dir
        :return:
        """
        import os
        return os.path.dirname(__file__)

saai_conf=SaaiConf()




import json
import logging
from typing import Any
from typing import Text, Optional, Dict

import questionary
from prompt_toolkit.styles import Style
from rasa.cli import utils as cli_utils
from rasa.core.channels import console
from rasa.core.interpreter import INTENT_MESSAGE_PREFIX

from saai.shell.settings import sett
import subprocess

logger = logging.getLogger(__name__)

DEFAULT_STREAM_READING_TIMEOUT_IN_SECONDS = 10


def _bot_output(
    message: Dict[Text, Any], color=cli_utils.bcolors.OKBLUE
) -> Optional[questionary.Question]:
    from sagas.nlu.tts_utils import say_lang
    # from sagas.kit.analysis_kit import AnalysisKit

    if ("text" in message) and not ("buttons" in message):
        text=message.get("text")
        cli_utils.print_color(text, color=color)
        say_lang(text, sett.lang, False)
        # AnalysisKit().console_vis(text, sett.lang)

    if "image" in message:
        cli_utils.print_color("Image: " + message.get("image"), color=color)

    if "attachment" in message:
        cli_utils.print_color("Attachment: " + message.get("attachment"), color=color)

    if "buttons" in message:
        choices = cli_utils.button_choices_from_message_data(
            message, allow_free_text_input=True
        )

        question = questionary.select(
            message.get("text"),
            choices,
            style=Style([("qmark", "#6d91d3"), ("", "#6d91d3"), ("answer", "#b373d6")]),
        )
        return question

    if "elements" in message:
        cli_utils.print_color("Elements:", color=color)
        for idx, element in enumerate(message.get("elements")):
            cli_utils.print_color(
                cli_utils.element_to_string(element, idx), color=color
            )

    if "quick_replies" in message:
        cli_utils.print_color("Quick Replies:", color=color)
        for idx, element in enumerate(message.get("quick_replies")):
            cli_utils.print_color(cli_utils.button_to_string(element, idx), color=color)

    if "custom" in message:
        cli_utils.print_color("Custom json:", color=color)
        cli_utils.print_color(json.dumps(message.get("custom"), indent=2), color=color)

def _user_input(button_question: questionary.Question) -> Optional[Text]:
    exit_text = INTENT_MESSAGE_PREFIX + "stop"

    if button_question is not None:
        response = cli_utils.payload_from_button_question(button_question)
        if response == cli_utils.FREE_TEXT_INPUT_PROMPT:
            # Re-prompt user with a free text input
            response = _user_input(None)
    else:
        response = questionary.text(
            "",
            qmark="Your input ->",
            style=Style([("qmark", "#b373d6"), ("", "#b373d6")]),
        ).ask()

    text= response.strip() if response is not None else None
    if not sett.is_disable_analyse:
        if text is not None and not text.startswith(INTENT_MESSAGE_PREFIX):
            subprocess.call(['sagas', 'vis', text, sett.lang])
    return text

def inject():
    console.print_bot_output=_bot_output
    console.get_user_input=_user_input

def shell_main():
    from rasa.__main__ import main
    from sagas.tool.loggers import init_logger

    init_logger()

    inject()
    main()

# $ python -m saai.shell shell
if __name__ == "__main__":
    shell_main()




import logging
logger = logging.getLogger(__name__)

class Settings(object):
    def __init__(self):
        from dotenv import load_dotenv
        from pathlib import Path
        import os
        env_path = Path('.') / '.env'
        load_dotenv(dotenv_path=env_path)
        self.lang=os.getenv("lang")
        self.disable_analyse=os.getenv('disable_analyse')
        print(f".. current lang {self.lang}, disable_analyse: {self.disable_analyse}")

    @property
    def is_disable_analyse(self):
        return self.disable_analyse is not None and self.disable_analyse=='true'

sett=Settings()

